@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # Primitivus: a SAT frontend
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -20,24 +20,27 @@
 from sat.core.i18n import _
 import urwid
 from urwid_satext import sat_widgets
+from sat_frontends.quick_frontend import quick_widgets
 
 
-class Progress(urwid.WidgetWrap):
+class Progress(urwid.WidgetWrap, quick_widgets.QuickWidget):
+    PROFILES_ALLOW_NONE = True
 
-    def __init__(self, host):
+    def __init__(self, host, target, profiles):
+        assert target is None and profiles is None
+        quick_widgets.QuickWidget.__init__(self, host, target)
         self.host = host
         self.progress_list = urwid.SimpleListWalker([])
         self.progress_dict = {}
         listbox = urwid.ListBox(self.progress_list)
         buttons = []
-        buttons.append(sat_widgets.CustomButton(_('Clear progress list'), self.__onClear))
+        buttons.append(sat_widgets.CustomButton(_('Clear progress list'), self._onClear))
         max_len = max([button.getSize() for button in buttons])
         buttons_wid = urwid.GridFlow(buttons,max_len,1,0,'center')
         main_wid = sat_widgets.FocusFrame(listbox, footer=buttons_wid)
         urwid.WidgetWrap.__init__(self, main_wid)
 
-    def addProgress(self, progress_id, message):
-        profile = self.host.profile # TODO: manage multiple profiles
+    def add(self, progress_id, message, profile):
         mess_wid = urwid.Text(message)
         progr_wid = urwid.ProgressBar('progress_normal', 'progress_complete')
         column = urwid.Columns([mess_wid, progr_wid])
@@ -47,8 +50,8 @@ class Progress(urwid.WidgetWrap):
 
     def progressCB(self, loop, data):
         progress_id, message, profile = data
-        data = self.host.bridge.getProgress(progress_id, profile)
-        pbar = self.progress_dict[(progress_id, profile)]['progress'] #FIXME: must manage profiles
+        data = self.host.bridge.progressGet(progress_id, profile)
+        pbar = self.progress_dict[(progress_id, profile)]['progress']
         if data:
             if self.progress_dict[(progress_id, profile)]['state'] == 'init':
                 #first answer, we must construct the bar
@@ -64,20 +67,20 @@ class Progress(urwid.WidgetWrap):
                 self.updateNotBar()
                 return
 
-        loop.set_alarm_in(1,self.progressCB, (progress_id, message, profile))
+        loop.set_alarm_in(0.2,self.progressCB, (progress_id, message, profile))
 
-    def __removeBar(self, progress_id, profile):
+    def _removeBar(self, progress_id, profile):
         wid = self.progress_dict[(progress_id, profile)]['full']
         self.progress_list.remove(wid)
         del(self.progress_dict[(progress_id, profile)])
 
-    def __onClear(self, button):
+    def _onClear(self, button):
        to_remove = []
        for progress_id, profile in self.progress_dict:
            if self.progress_dict[(progress_id, profile)]['state'] == 'done':
                to_remove.append((progress_id, profile))
        for progress_id, profile in to_remove:
-           self.__removeBar(progress_id, profile)
+           self._removeBar(progress_id, profile)
        self.updateNotBar()
 
     def updateNotBar(self):
