@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SàT frontend tools
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -20,7 +20,7 @@
 from sat.core.i18n import _
 from sat.core.log import getLogger
 log = getLogger(__name__)
-from sat_frontends.constants import Const as C
+from sat_frontends.quick_frontend.constants import Const as C
 from sat.core.exceptions import DataError
 
 
@@ -35,6 +35,8 @@ class InvalidXMLUI(Exception):
 class ClassNotRegistedError(Exception):
     pass
 
+
+# FIXME: this method is duplicated in frontends.tools.xmlui.getText
 def getText(node):
     """Get child text nodes
     @param node: dom Node
@@ -83,30 +85,54 @@ class StringWidget(Widget):
 
     often called Edit in toolkits
     """
+    pass
+
+
+class JidInputWidget(Widget):
+    """Input widget wich require a string
+
+    often called Edit in toolkits
+    """
+    pass
 
 
 class PasswordWidget(Widget):
     """Input widget with require a masked string"""
+    pass
 
 
 class TextBoxWidget(Widget):
     """Input widget with require a long, possibly multilines string
     often called TextArea in toolkits
     """
+    pass
 
 
 class BoolWidget(Widget):
     """Input widget with require a boolean value
     often called CheckBox in toolkits
     """
+    pass
+
+
+class IntWidget(Widget):
+    """Input widget with require an integer"""
+    pass
 
 
 class ButtonWidget(Widget):
     """A clickable widget"""
+    pass
 
 
 class ListWidget(Widget):
     """A widget able to show/choose one or several strings in a list"""
+    pass
+
+
+class JidsListWidget(Widget):
+    """A widget able to show/choose one or several strings in a list"""
+    pass
 
 
 class Container(Widget):
@@ -125,6 +151,7 @@ class Container(Widget):
 
 class PairsContainer(Container):
     """Widgets are disposed in rows of two (usually label/input) """
+    pass
 
 
 class TabsContainer(Container):
@@ -132,13 +159,16 @@ class TabsContainer(Container):
 
     Often called Notebook in toolkits
     """
+    pass
 
 class VerticalContainer(Container):
     """Widgets are disposed vertically"""
+    pass
 
 
 class AdvancedListContainer(Container):
     """Widgets are disposed in rows with advaned features"""
+    pass
 
 
 class Dialog(object):
@@ -152,13 +182,11 @@ class Dialog(object):
             data = {}
         self._xmluiSetData(C.XMLUI_STATUS_VALIDATED, data)
         self._xmluiSubmit(data)
-        self._xmluiClose()
 
     def _xmluiCancelled(self):
         data = {C.XMLUI_DATA_CANCELLED: C.BOOL_TRUE}
         self._xmluiSetData(C.XMLUI_STATUS_CANCELLED, data)
         self._xmluiSubmit(data)
-        self._xmluiClose()
 
     def _xmluiSubmit(self, data):
         if self._xmlui_parent.submit_id is None:
@@ -172,10 +200,12 @@ class Dialog(object):
 
 class MessageDialog(Dialog):
     """Dialog with a OK/Cancel type configuration"""
+    pass
 
 
 class NoteDialog(Dialog):
     """Dialog with a OK/Cancel type configuration"""
+    pass
 
 
 class ConfirmDialog(Dialog):
@@ -190,6 +220,7 @@ class ConfirmDialog(Dialog):
 
 class FileDialog(Dialog):
     """Dialog with a OK/Cancel type configuration"""
+    pass
 
 
 class XMLUIBase(object):
@@ -198,7 +229,7 @@ class XMLUIBase(object):
     This class must not be instancied directly
     """
 
-    def __init__(self, host, parsed_dom, title = None, flags = None):
+    def __init__(self, host, parsed_dom, title=None, flags=None, callback=None, profile=C.PROF_KEY_NONE):
         """Initialise the XMLUI instance
 
         @param host: %(doc_host)s
@@ -206,15 +237,27 @@ class XMLUIBase(object):
         @param title: force the title, or use XMLUI one if None
         @param flags: list of string which can be:
             - NO_CANCEL: the UI can't be cancelled
+        @param callback(callable, None): if not None, will be used with launchAction:
+            - if None is used, default behaviour will be used (closing the dialog and calling host.actionManager)
+            - if a callback is provided, it will be used instead, so you'll have to manage
+                dialog closing or new xmlui to display, or other action (you can call host.actionManager)
         """
         self.host = host
         top=parsed_dom.documentElement
         self.session_id = top.getAttribute("session_id") or None
         self.submit_id = top.getAttribute("submit") or None
-        self.title = title or top.getAttribute("title") or u""
+        self.xmlui_title = title or top.getAttribute("title") or u""
         if flags is None:
             flags = []
         self.flags = flags
+        self.callback = callback or self._defaultCb
+        self.profile = profile
+
+    def _defaultCb(self, data, cb_id, profile):
+        # TODO: when XMLUI updates will be managed, the _xmluiClose
+        #       must be called only if there is not update
+        self._xmluiClose()
+        self.host.actionManager(data, profile=profile)
 
     def _isAttrSet(self, name, node):
         """Returnw widget boolean attribute status
@@ -240,6 +283,7 @@ class XMLUIBase(object):
         return None
 
     def submit(self, data):
+        self._xmluiClose()
         if self.submit_id is None:
             raise ValueError("Can't submit is self.submit_id is not set")
         if "session_id" in data:
@@ -249,7 +293,14 @@ class XMLUIBase(object):
         self._xmluiLaunchAction(self.submit_id, data)
 
     def _xmluiLaunchAction(self, action_id, data):
-        self.host.launchAction(action_id, data, profile_key = self.host.profile)
+        self.host.launchAction(action_id, data, callback=self.callback, profile=self.profile)
+
+    def _xmluiClose(self):
+        """Close the window/popup/... where the constructor XMLUI is
+
+        this method must be overrided
+        """
+        raise NotImplementedError
 
 
 class XMLUIPanel(XMLUIBase):
@@ -257,12 +308,12 @@ class XMLUIPanel(XMLUIBase):
 
     New frontends can inherite this class to easily implement XMLUI
     @property widget_factory: factory to create frontend-specific widgets
-    @proporety dialog_factory: factory to create frontend-specific dialogs
+    @property dialog_factory: factory to create frontend-specific dialogs
     """
     widget_factory = None
 
-    def __init__(self, host, parsed_dom, title = None, flags = None):
-        super(XMLUIPanel, self).__init__(host, parsed_dom, title = None, flags = None)
+    def __init__(self, host, parsed_dom, title=None, flags=None, callback=None, profile=C.PROF_KEY_NONE):
+        super(XMLUIPanel, self).__init__(host, parsed_dom, title=title, flags=flags, callback=callback, profile=profile)
         self.ctrl_list = {}  # usefull to access ctrl
         self._main_cont = None
         self.constructUI(parsed_dom)
@@ -282,7 +333,7 @@ class XMLUIPanel(XMLUIBase):
         self._main_cont = value
 
     def _parseChilds(self, _xmlui_parent, current_node, wanted = ('container',), data = None):
-        """Recursively parse childNodes of an elemen
+        """Recursively parse childNodes of an element
 
         @param _xmlui_parent: widget container with '_xmluiAppend' method
         @param current_node: element from which childs will be parsed
@@ -340,12 +391,13 @@ class XMLUIPanel(XMLUIBase):
             elif node.nodeName == 'tab':
                 name = node.getAttribute('name')
                 label = node.getAttribute('label')
+                selected = C.bool(node.getAttribute('selected') or C.BOOL_FALSE)
                 if not name or not isinstance(data, TabsContainer):
                     raise InvalidXMLUI
                 if self.type == 'param':
                     self._current_category = name #XXX: awful hack because params need category and we don't keep parent
                 tab_cont = data
-                new_tab = tab_cont._xmluiAddTab(label or name)
+                new_tab = tab_cont._xmluiAddTab(label or name, selected)
                 self._parseChilds(new_tab, node, ('widget', 'container'))
 
             elif node.nodeName == 'row':
@@ -379,6 +431,9 @@ class XMLUIPanel(XMLUIBase):
                 elif type_=="string":
                     ctrl = self.widget_factory.createStringWidget(_xmlui_parent, value, self._isAttrSet("read_only", node))
                     self.ctrl_list[name] = ({'type':type_, 'control':ctrl})
+                elif type_=="jid_input":
+                    ctrl = self.widget_factory.createJidInputWidget(_xmlui_parent, value, self._isAttrSet("read_only", node))
+                    self.ctrl_list[name] = ({'type':type_, 'control':ctrl})
                 elif type_=="password":
                     ctrl = self.widget_factory.createPasswordWidget(_xmlui_parent, value, self._isAttrSet("read_only", node))
                     self.ctrl_list[name] = ({'type':type_, 'control':ctrl})
@@ -388,12 +443,20 @@ class XMLUIPanel(XMLUIBase):
                 elif type_=="bool":
                     ctrl = self.widget_factory.createBoolWidget(_xmlui_parent, value==C.BOOL_TRUE, self._isAttrSet("read_only", node))
                     self.ctrl_list[name] = ({'type':type_, 'control':ctrl})
+                elif type_ == "int":
+                    ctrl = self.widget_factory.createIntWidget(_xmlui_parent, value, self._isAttrSet("read_only", node))
+                    self.ctrl_list[name] = ({'type':type_, 'control':ctrl})
                 elif type_ == "list":
                     style = [] if node.getAttribute("multi") == 'yes' else ['single']
                     _options = [(option.getAttribute("value"), option.getAttribute("label")) for option in node.getElementsByTagName("option")]
                     _selected = [option.getAttribute("value") for option in node.getElementsByTagName("option") if option.getAttribute('selected') == C.BOOL_TRUE]
                     ctrl = self.widget_factory.createListWidget(_xmlui_parent, _options, _selected, style)
                     self.ctrl_list[name] = ({'type': type_, 'control': ctrl})
+                elif type_ == "jids_list":
+                    style = []
+                    jids = [getText(jid_) for jid_ in node.getElementsByTagName("jid")]
+                    ctrl = self.widget_factory.createJidsListWidget(_xmlui_parent, jids, style)
+                    self.ctrl_list[name] = ({'type': type_, 'control': ctrl})
                 elif type_=="button":
                     callback_id = node.getAttribute("callback")
                     ctrl = self.widget_factory.createButtonWidget(_xmlui_parent, value, self.onButtonPress)
@@ -410,7 +473,7 @@ class XMLUIPanel(XMLUIBase):
                         if not isinstance(ctrl, (EmptyWidget, TextWidget, LabelWidget, JidWidget)):
                             log.warning(_("No change listener on [%s]") % ctrl)
 
-                if type_ != 'text':
+                elif type_ != 'text':
                     callback = node.getAttribute("internal_callback") or None
                     if callback:
                         fields = [field.getAttribute('name') for field in node.getElementsByTagName("internal_field")]
@@ -447,15 +510,8 @@ class XMLUIPanel(XMLUIBase):
         if post_treat is not None:
             post_treat()
 
-    def _xmluiClose(self):
-        """Close the window/popup/... where the constructeur XMLUI is
-
-        this method must be overrided
-        """
-        raise NotImplementedError
-
     def _xmluiSetParam(self, name, value, category):
-        self.host.bridge.setParam(name, value, category, profile_key=self.host.profile)
+        self.host.bridge.setParam(name, value, category, profile_key=self.profile)
 
     ##EVENTS##
 
@@ -464,7 +520,7 @@ class XMLUIPanel(XMLUIBase):
 
         @param ctrl: widget modified
         """
-        assert(self.type == "param")
+        assert self.type == "param"
         self.param_changed.add(ctrl)
 
     def onAdvListSelect(self, ctrl):
@@ -539,8 +595,8 @@ class XMLUIPanel(XMLUIBase):
 
         def groups_of_contact(source, target):
             """Select in target the groups of the contact which is selected in source."""
-            assert(isinstance(source, ListWidget))
-            assert(isinstance(target, ListWidget))
+            assert isinstance(source, ListWidget)
+            assert isinstance(target, ListWidget)
             try:
                 contact_jid_s = source._xmluiGetSelectedValues()[0]
             except IndexError:
@@ -602,7 +658,7 @@ class XMLUIPanel(XMLUIBase):
             self.submit(data)
         else:
             log.warning(_("The form data is not sent back, the type is not managed properly"))
-        self._xmluiClose()
+            self._xmluiClose()
 
     def onFormCancelled(self, ignore=None):
         """Called when a form is cancelled"""
@@ -614,7 +670,7 @@ class XMLUIPanel(XMLUIBase):
 
         self.type must be param
         """
-        assert(self.type == 'param')
+        assert self.type == 'param'
         for ctrl in self.param_changed:
             if isinstance(ctrl, ListWidget):
                 value = u'\t'.join(ctrl._xmluiGetSelectedValues())
@@ -632,8 +688,8 @@ class XMLUIPanel(XMLUIBase):
 class XMLUIDialog(XMLUIBase):
     dialog_factory = None
 
-    def __init__(self, host, parsed_dom, title = None, flags = None):
-        super(XMLUIDialog, self).__init__(host, parsed_dom, title = None, flags = None)
+    def __init__(self, host, parsed_dom, title=None, flags=None, callback=None, profile=C.PROF_KEY_NONE):
+        super(XMLUIDialog, self).__init__(host, parsed_dom, title=title, flags=flags, callback=callback, profile=profile)
         top=parsed_dom.documentElement
         dlg_elt =  self._getChildNode(top, "dialog")
         if dlg_elt is None:
@@ -647,29 +703,32 @@ class XMLUIDialog(XMLUIBase):
         level = dlg_elt.getAttribute(C.XMLUI_DATA_LVL) or C.XMLUI_DATA_LVL_INFO
 
         if dlg_type == C.XMLUI_DIALOG_MESSAGE:
-            self.dlg = self.dialog_factory.createMessageDialog(self, self.title, message, level)
+            self.dlg = self.dialog_factory.createMessageDialog(self, self.xmlui_title, message, level)
         elif dlg_type == C.XMLUI_DIALOG_NOTE:
-            self.dlg = self.dialog_factory.createNoteDialog(self, self.title, message, level)
+            self.dlg = self.dialog_factory.createNoteDialog(self, self.xmlui_title, message, level)
         elif dlg_type == C.XMLUI_DIALOG_CONFIRM:
             try:
                 buttons_elt = self._getChildNode(dlg_elt, "buttons")
                 buttons_set = buttons_elt.getAttribute("set") or C.XMLUI_DATA_BTNS_SET_DEFAULT
             except (TypeError, AttributeError): # XXX: TypeError is here because pyjamas raise a TypeError instead of an AttributeError
                 buttons_set = C.XMLUI_DATA_BTNS_SET_DEFAULT
-            self.dlg = self.dialog_factory.createConfirmDialog(self, self.title, message, level, buttons_set)
+            self.dlg = self.dialog_factory.createConfirmDialog(self, self.xmlui_title, message, level, buttons_set)
         elif dlg_type == C.XMLUI_DIALOG_FILE:
             try:
                 file_elt = self._getChildNode(dlg_elt, "file")
                 filetype = file_elt.getAttribute("type") or C.XMLUI_DATA_FILETYPE_DEFAULT
             except (TypeError, AttributeError): # XXX: TypeError is here because pyjamas raise a TypeError instead of an AttributeError
                 filetype = C.XMLUI_DATA_FILETYPE_DEFAULT
-            self.dlg = self.dialog_factory.createFileDialog(self, self.title, message, level, filetype)
+            self.dlg = self.dialog_factory.createFileDialog(self, self.xmlui_title, message, level, filetype)
         else:
             raise ValueError("Unknown dialog type [%s]" % dlg_type)
 
     def show(self):
         self.dlg._xmluiShow()
 
+    def _xmluiClose(self):
+        self.dlg._xmluiClose()
+
 
 def registerClass(type_, class_):
     """Register the class to use with the factory
@@ -683,9 +742,9 @@ def registerClass(type_, class_):
     class_map[type_] = class_
 
 
-def create(host, xml_data, title = None, flags = None, dom_parse=None, dom_free=None):
+def create(host, xml_data, title=None, flags=None, dom_parse=None, dom_free=None, callback=None, profile=C.PROF_KEY_NONE):
     """
-        @param dom_parse: methode equivalent to minidom.parseString (but which  must manage unicode), or None to use default one
+        @param dom_parse: methode equivalent to minidom.parseString (but which must manage unicode), or None to use default one
         @param dom_free: method used to free the parsed DOM
     """
     if dom_parse is None:
@@ -706,6 +765,6 @@ def create(host, xml_data, title = None, flags = None, dom_parse=None, dom_free=
     except KeyError:
         raise ClassNotRegistedError(_("You must register classes with registerClass before creating a XMLUI"))
 
-    xmlui = cls(host, parsed_dom, title, flags)
+    xmlui = cls(host, parsed_dom, title, flags, callback, profile)
     dom_free(parsed_dom)
     return xmlui
