@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 #-*- coding: utf-8 -*-
 
 # SAT: a jabber client
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -125,6 +125,11 @@ class DbusObject(dbus.service.Object):
         #       further investigations
         pass
 
+    @dbus.service.signal(const_INT_PREFIX+const_CORE_SUFFIX,
+                         signature='a{ss}sis')
+    def actionNew(self, action_data, id, security_limit, profile):
+        pass
+
     @dbus.service.signal(const_INT_PREFIX+const_CORE_SUFFIX,
                          signature='ssa{ss}s')
     def actionResult(self, answer_type, id, data, profile):
@@ -141,8 +146,8 @@ class DbusObject(dbus.service.Object):
         pass
 
     @dbus.service.signal(const_INT_PREFIX+const_CORE_SUFFIX,
-                         signature='s')
-    def connected(self, profile):
+                         signature='ss')
+    def connected(self, profile, jid_s):
         pass
 
     @dbus.service.signal(const_INT_PREFIX+const_CORE_SUFFIX,
@@ -185,6 +190,21 @@ class DbusObject(dbus.service.Object):
     def presenceUpdate(self, entity_jid, show, priority, statuses, profile):
         pass
 
+    @dbus.service.signal(const_INT_PREFIX+const_CORE_SUFFIX,
+                         signature='sss')
+    def progressError(self, id, error, profile):
+        pass
+
+    @dbus.service.signal(const_INT_PREFIX+const_CORE_SUFFIX,
+                         signature='sa{ss}s')
+    def progressFinished(self, id, metadata, profile):
+        pass
+
+    @dbus.service.signal(const_INT_PREFIX+const_CORE_SUFFIX,
+                         signature='sa{ss}s')
+    def progressStarted(self, id, metadata, profile):
+        pass
+
     @dbus.service.signal(const_INT_PREFIX+const_CORE_SUFFIX,
                          signature='sss')
     def subscribe(self, sub_type, entity_jid, profile):
@@ -192,6 +212,12 @@ class DbusObject(dbus.service.Object):
 
     ### methods ###
 
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='s', out_signature='a(a{ss}si)',
+                         async_callbacks=None)
+    def actionsGet(self, profile_key="@DEFAULT@"):
+        return self._callback("actionsGet", unicode(profile_key))
+
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ss', out_signature='',
                          async_callbacks=None)
@@ -222,6 +248,12 @@ class DbusObject(dbus.service.Object):
     def asyncGetParamA(self, name, category, attribute="value", security_limit=-1, profile_key="@DEFAULT@", callback=None, errback=None):
         return self._callback("asyncGetParamA", unicode(name), unicode(category), unicode(attribute), security_limit, unicode(profile_key), callback=callback, errback=errback)
 
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='sis', out_signature='a{ss}',
+                         async_callbacks=('callback', 'errback'))
+    def asyncGetParamsValuesFromCategory(self, category, security_limit=-1, profile_key="@DEFAULT@", callback=None, errback=None):
+        return self._callback("asyncGetParamsValuesFromCategory", unicode(category), security_limit, unicode(profile_key), callback=callback, errback=errback)
+
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='sba{ss}s', out_signature='',
                          async_callbacks=None)
@@ -230,12 +262,12 @@ class DbusObject(dbus.service.Object):
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ss', out_signature='',
-                         async_callbacks=None)
-    def delContact(self, entity_jid, profile_key="@DEFAULT@"):
-        return self._callback("delContact", unicode(entity_jid), unicode(profile_key))
+                         async_callbacks=('callback', 'errback'))
+    def delContact(self, entity_jid, profile_key="@DEFAULT@", callback=None, errback=None):
+        return self._callback("delContact", unicode(entity_jid), unicode(profile_key), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='ss', out_signature='(asa(sss))',
+                         in_signature='ss', out_signature='(asa(sss)a{sa(a{ss}as)})',
                          async_callbacks=('callback', 'errback'))
     def discoInfos(self, entity_jid, profile_key, callback=None, errback=None):
         return self._callback("discoInfos", unicode(entity_jid), unicode(profile_key), callback=callback, errback=errback)
@@ -260,9 +292,9 @@ class DbusObject(dbus.service.Object):
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='s', out_signature='a(sa{ss}as)',
-                         async_callbacks=None)
-    def getContacts(self, profile_key="@DEFAULT@"):
-        return self._callback("getContacts", unicode(profile_key))
+                         async_callbacks=('callback', 'errback'))
+    def getContacts(self, profile_key="@DEFAULT@", callback=None, errback=None):
+        return self._callback("getContacts", unicode(profile_key), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ss', out_signature='as',
@@ -270,6 +302,12 @@ class DbusObject(dbus.service.Object):
     def getContactsFromGroup(self, group, profile_key="@DEFAULT@"):
         return self._callback("getContactsFromGroup", unicode(group), unicode(profile_key))
 
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='asass', out_signature='a{sa{ss}}',
+                         async_callbacks=None)
+    def getEntitiesData(self, jids, keys, profile):
+        return self._callback("getEntitiesData", jids, keys, unicode(profile))
+
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='sass', out_signature='a{ss}',
                          async_callbacks=None)
@@ -277,16 +315,22 @@ class DbusObject(dbus.service.Object):
         return self._callback("getEntityData", unicode(jid), keys, unicode(profile))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='ssibs', out_signature='a(dssssa{ss})',
+                         in_signature='s', out_signature='a{sa{ss}}',
                          async_callbacks=('callback', 'errback'))
-    def getHistory(self, from_jid, to_jid, limit, between=True, profile="@NONE@", callback=None, errback=None):
-        return self._callback("getHistory", unicode(from_jid), unicode(to_jid), limit, between, unicode(profile), callback=callback, errback=errback)
+    def getFeatures(self, profile_key, callback=None, errback=None):
+        return self._callback("getFeatures", unicode(profile_key), callback=callback, errback=errback)
+
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='ssibss', out_signature='a(dssssa{ss})',
+                         async_callbacks=('callback', 'errback'))
+    def getHistory(self, from_jid, to_jid, limit, between=True, search='', profile="@NONE@", callback=None, errback=None):
+        return self._callback("getHistory", unicode(from_jid), unicode(to_jid), limit, between, unicode(search), unicode(profile), callback=callback, errback=errback)
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ss', out_signature='s',
                          async_callbacks=None)
-    def getLastResource(self, contact_jid, profile_key="@DEFAULT@"):
-        return self._callback("getLastResource", unicode(contact_jid), unicode(profile_key))
+    def getMainResource(self, contact_jid, profile_key="@DEFAULT@"):
+        return self._callback("getMainResource", unicode(contact_jid), unicode(profile_key))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ss', out_signature='s',
@@ -295,7 +339,7 @@ class DbusObject(dbus.service.Object):
         return self._callback("getMenuHelp", unicode(menu_id), unicode(language))
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='si', out_signature='a(ssasas)',
+                         in_signature='si', out_signature='a(ssasasa{ss})',
                          async_callbacks=None)
     def getMenus(self, language, security_limit):
         return self._callback("getMenus", unicode(language), security_limit)
@@ -306,24 +350,12 @@ class DbusObject(dbus.service.Object):
     def getParamA(self, name, category, attribute="value", profile_key="@DEFAULT@"):
         return self._callback("getParamA", unicode(name), unicode(category), unicode(attribute), unicode(profile_key))
 
-    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='iss', out_signature='s',
-                         async_callbacks=('callback', 'errback'))
-    def getParams(self, security_limit=-1, app='', profile_key="@DEFAULT@", callback=None, errback=None):
-        return self._callback("getParams", security_limit, unicode(app), unicode(profile_key), callback=callback, errback=errback)
-
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='', out_signature='as',
                          async_callbacks=None)
     def getParamsCategories(self, ):
         return self._callback("getParamsCategories", )
 
-    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='siss', out_signature='s',
-                         async_callbacks=('callback', 'errback'))
-    def getParamsForCategory(self, category, security_limit=-1, app='', profile_key="@DEFAULT@", callback=None, errback=None):
-        return self._callback("getParamsForCategory", unicode(category), security_limit, unicode(app), unicode(profile_key), callback=callback, errback=errback)
-
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='iss', out_signature='s',
                          async_callbacks=('callback', 'errback'))
@@ -348,12 +380,6 @@ class DbusObject(dbus.service.Object):
     def getProfilesList(self, ):
         return self._callback("getProfilesList", )
 
-    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
-                         in_signature='ss', out_signature='a{ss}',
-                         async_callbacks=None)
-    def getProgress(self, id, profile):
-        return self._callback("getProgress", unicode(id), unicode(profile))
-
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='', out_signature='',
                          async_callbacks=('callback', 'errback'))
@@ -402,6 +428,42 @@ class DbusObject(dbus.service.Object):
     def paramsRegisterApp(self, xml, security_limit=-1, app=''):
         return self._callback("paramsRegisterApp", unicode(xml), security_limit, unicode(app))
 
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='s', out_signature='b',
+                         async_callbacks=None)
+    def profileIsSessionStarted(self, profile_key="@DEFAULT@"):
+        return self._callback("profileIsSessionStarted", unicode(profile_key))
+
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='s', out_signature='',
+                         async_callbacks=None)
+    def profileSetDefault(self, arg_0):
+        return self._callback("profileSetDefault", unicode(arg_0))
+
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='ss', out_signature='b',
+                         async_callbacks=('callback', 'errback'))
+    def profileStartSession(self, password='', profile_key="@DEFAULT@", callback=None, errback=None):
+        return self._callback("profileStartSession", unicode(password), unicode(profile_key), callback=callback, errback=errback)
+
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='ss', out_signature='a{ss}',
+                         async_callbacks=None)
+    def progressGet(self, id, profile):
+        return self._callback("progressGet", unicode(id), unicode(profile))
+
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='s', out_signature='a{sa{sa{ss}}}',
+                         async_callbacks=None)
+    def progressGetAll(self, profile):
+        return self._callback("progressGetAll", unicode(profile))
+
+    @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
+                         in_signature='s', out_signature='a{sa{sa{ss}}}',
+                         async_callbacks=None)
+    def progressGetAllMetadata(self, profile):
+        return self._callback("progressGetAllMetadata", unicode(profile))
+
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='s', out_signature='b',
                          async_callbacks=None)
@@ -434,9 +496,9 @@ class DbusObject(dbus.service.Object):
 
     @dbus.service.method(const_INT_PREFIX+const_CORE_SUFFIX,
                          in_signature='ssass', out_signature='',
-                         async_callbacks=None)
-    def updateContact(self, entity_jid, name, groups, profile_key="@DEFAULT@"):
-        return self._callback("updateContact", unicode(entity_jid), unicode(name), groups, unicode(profile_key))
+                         async_callbacks=('callback', 'errback'))
+    def updateContact(self, entity_jid, name, groups, profile_key="@DEFAULT@", callback=None, errback=None):
+        return self._callback("updateContact", unicode(entity_jid), unicode(name), groups, unicode(profile_key), callback=callback, errback=errback)
 
     def __attributes(self, in_sign):
         """Return arguments to user given a in_sign
@@ -540,6 +602,9 @@ class DBusBridge(Bridge):
         self.dbus_name = dbus.service.BusName(const_INT_PREFIX, self.session_bus)
         self.dbus_bridge = DbusObject(self.session_bus, const_OBJ_PATH)
 
+    def actionNew(self, action_data, id, security_limit, profile):
+        self.dbus_bridge.actionNew(action_data, id, security_limit, profile)
+
     def actionResult(self, answer_type, id, data, profile):
         self.dbus_bridge.actionResult(answer_type, id, data, profile)
 
@@ -549,8 +614,8 @@ class DBusBridge(Bridge):
     def askConfirmation(self, id, conf_type, data, profile):
         self.dbus_bridge.askConfirmation(id, conf_type, data, profile)
 
-    def connected(self, profile):
-        self.dbus_bridge.connected(profile)
+    def connected(self, profile, jid_s):
+        self.dbus_bridge.connected(profile, jid_s)
 
     def contactDeleted(self, entity_jid, profile):
         self.dbus_bridge.contactDeleted(entity_jid, profile)
@@ -576,6 +641,15 @@ class DBusBridge(Bridge):
     def presenceUpdate(self, entity_jid, show, priority, statuses, profile):
         self.dbus_bridge.presenceUpdate(entity_jid, show, priority, statuses, profile)
 
+    def progressError(self, id, error, profile):
+        self.dbus_bridge.progressError(id, error, profile)
+
+    def progressFinished(self, id, metadata, profile):
+        self.dbus_bridge.progressFinished(id, metadata, profile)
+
+    def progressStarted(self, id, metadata, profile):
+        self.dbus_bridge.progressStarted(id, metadata, profile)
+
     def subscribe(self, sub_type, entity_jid, profile):
         self.dbus_bridge.subscribe(sub_type, entity_jid, profile)
 
