@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 #-*- coding: utf-8 -*-
 
 # SAT: a jabber client
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -446,16 +446,35 @@ class DbusConstructor(Constructor):
                 'args': self.getArguments(function['sig_in'], name=arg_doc, default=default)}
 
             if function["type"] == "method":
+                # XXX: we can manage blocking call in the same way as async one: if callback is None the call will be blocking
                 completion['debug'] = "" if not self.options.debug else 'log.debug ("%s")\n%s' % (section, 8 * ' ')
                 completion['args_result'] = self.getArguments(function['sig_in'], name=arg_doc)
-                completion['async_args'] = 'callback=None, errback=None' if async else ''
-                completion['async_comma'] = ', ' if async and function['sig_in'] else ''
-                completion['async_args_result'] = 'timeout=const_TIMEOUT, reply_handler=callback, error_handler=lambda err:errback(dbus_to_bridge_exception(err))' if async else ''
+                completion['async_args'] = 'callback=None, errback=None'
+                completion['async_comma'] = ', ' if function['sig_in'] else ''
+                completion['error_handler'] = """if callback is None:
+            error_handler = None
+        else:
+            if errback is None:
+                errback = log.error
+            error_handler = lambda err:errback(dbus_to_bridge_exception(err))
+        """
+                if async:
+                    completion['blocking_call'] = ''
+                    completion['async_args_result'] = 'timeout=const_TIMEOUT, reply_handler=callback, error_handler=error_handler'
+                else:
+                    # XXX: To have a blocking call, we must have not reply_handler, so we test if callback exists, and add reply_handler only in this case
+                    completion['blocking_call'] = """kwargs={}
+        if callback is not None:
+            kwargs['timeout'] = const_TIMEOUT
+            kwargs['reply_handler'] = callback
+            kwargs['error_handler'] = error_handler
+        """
+                    completion['async_args_result'] = '**kwargs'
                 result = "self.db_%(category)s_iface.%(name)s(%(args_result)s%(async_comma)s%(async_args_result)s)" % completion
                 completion['result'] = ("unicode(%s)" if self.options.unicode and function['sig_out'] == 's' else "%s") % result
                 methods_part.append("""\
     def %(name)s(self, %(args)s%(async_comma)s%(async_args)s):
-        %(debug)sreturn %(result)s
+        %(error_handler)s%(blocking_call)s%(debug)sreturn %(result)s
 """ % completion)
 
         #at this point, methods_part should be filled,
