@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SàT Exceptions
@@ -33,16 +33,16 @@ class ProfileNotSetError(Exception):
     pass
 
 
-class ConnectedProfileError(Exception):
+class ProfileConnected(Exception):
     """This error is raised when trying to delete a connected profile."""
     pass
 
 
-class NotConnectedProfileError(Exception):
+class ProfileNotConnected(Exception):
     pass
 
 
-class ProfileKeyUnknownError(Exception):
+class ProfileKeyUnknown(Exception):
     pass
 
 
@@ -54,6 +54,12 @@ class UnknownGroupError(Exception):
     pass
 
 
+class MissingModule(Exception):
+    # Used to indicate when a plugin dependence is not found
+    # it's nice to indicate when to find the dependence in argument string
+    pass
+
+
 class NotFound(Exception):
     pass
 
@@ -66,6 +72,10 @@ class ConflictError(Exception):
     pass
 
 
+class TimeOutError(Exception):
+    pass
+
+
 class CancelError(Exception):
     pass
 
@@ -93,5 +103,14 @@ class DatabaseError(Exception):
 class PasswordError(Exception):
     pass
 
+
+class PermissionError(Exception):
+    pass
+
+
 class SkipHistory(Exception): # used in MessageReceivedTrigger to avoid history writting
     pass
+
+
+class ParsingError(Exception):
+    pass
