@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT: a jabber client
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -21,37 +21,55 @@ from sat.core.i18n import _
 from sat.core.constants import Const as C
 from twisted.internet import task, defer
 from twisted.words.protocols.jabber import jid, xmlstream
-from wokkel import client, disco, xmppim, generic, compat, delay, iwokkel
+from twisted.words.protocols.jabber import error
+from wokkel import client, disco, xmppim, generic, delay, iwokkel
 from sat.core.log import getLogger
 log = getLogger(__name__)
 from sat.core import exceptions
-from calendar import timegm
 from zope.interface import implements
-try:
-    from twisted.words.protocols.xmlstream import XMPPHandler
-except ImportError:
-    from wokkel.subprotocols import XMPPHandler
+from twisted.words.protocols.jabber.xmlstream import XMPPHandler
 
 
 class SatXMPPClient(client.XMPPClient):
     implements(iwokkel.IDisco)
 
-    def __init__(self, host_app, profile, user_jid, password, host=None, port=C.XMPP_C2S_PORT):
+    def __init__(self, host_app, profile, user_jid, password, host=None, port=C.XMPP_C2S_PORT, max_retries=C.XMPP_MAX_RETRIES):
         # XXX: DNS SRV records are checked when the host is not specified.
         # If no SRV record is found, the host is directly extracted from the JID.
         client.XMPPClient.__init__(self, user_jid, password, host or None, port or C.XMPP_C2S_PORT)
         self.factory.clientConnectionLost = self.connectionLost
+        self.factory.maxRetries = max_retries
         self.__connected = False
         self.profile = profile
         self.host_app = host_app
         self.conn_deferred = defer.Deferred()
         self._waiting_conf = {}  # callback called when a confirmation is received
-        self._progress_cb_map = {}  # callback called when a progress is requested (key = progress id)
+        self._progress_cb = {}  # callback called when a progress is requested (key = progress id)
+        self.actions = {} # used to keep track of actions for retrieval (key = action_id)
 
     def getConnectionDeferred(self):
         """Return a deferred which fire when the client is connected"""
         return self.conn_deferred
 
+    def IQ(self, type_=u'set', timeout=None):
+        """shortcut to create an IQ element managing deferred
+
+        @param type_(unicode): IQ type ('set' or 'get')
+        @param timeout(None, int): timeout in seconds
+        """
+        iq_elt = xmlstream.IQ(self.xmlstream, type_)
+        iq_elt.timeout = timeout
+        return iq_elt
+
+    def sendError(self, iq_elt, condition):
+        """Send error stanza build from iq_elt
+
+        @param iq_elt(domish.Element): initial IQ element
+        @param condition(unicode): error condition
+        """
+        iq_error_elt = error.StanzaError(condition).toResponse(iq_elt)
+        self.xmlstream.send(iq_error_elt)
+
     def _authd(self, xmlstream):
         if not self.host_app.trigger.point("XML Initialized", xmlstream, self.profile):
             return
@@ -59,13 +77,13 @@ class SatXMPPClient(client.XMPPClient):
         self.__connected = True
         log.info(_("********** [%s] CONNECTED **********") % self.profile)
         self.streamInitialized()
-        self.host_app.bridge.connected(self.profile)  # we send the signal to the clients
+        self.host_app.bridge.connected(self.profile, unicode(self.jid))  # we send the signal to the clients
 
     def streamInitialized(self):
         """Called after _authd"""
         log.debug(_("XML stream is initialized"))
         self.keep_alife = task.LoopingCall(self.xmlstream.send, " ")  # Needed to avoid disconnection (specially with openfire)
-        self.keep_alife.start(180)
+        self.keep_alife.start(C.XMPP_KEEP_ALIFE)
 
         self.disco = SatDiscoProtocol(self)
         self.disco.setHandlerParent(self)
@@ -84,7 +102,7 @@ class SatXMPPClient(client.XMPPClient):
         disco_d.addCallback(finish_connection)
 
     def initializationFailed(self, reason):
-        log.error(_("ERROR: XMPP connection failed for profile '%(profile)s': %(reason)s" % {'profile': self.profile, 'reason': reason}))
+        log.error(_(u"ERROR: XMPP connection failed for profile '%(profile)s': %(reason)s" % {'profile': self.profile, 'reason': reason}))
         self.conn_deferred.errback(reason.value)
         try:
             client.XMPPClient.initializationFailed(self, reason)
@@ -114,6 +132,8 @@ class SatMessageProtocol(xmppim.MessageProtocol):
         self.host = host
 
     def onMessage(self, message):
+        if not message.hasAttribute('from'):
+            message['from'] = self.parent.jid.host
         log.debug(_(u"got message from: %s") % message["from"])
         post_treat = defer.Deferred() # XXX: plugin can add their treatments to this deferred
 
@@ -139,17 +159,15 @@ class SatMessageProtocol(xmppim.MessageProtocol):
             return data
 
         def addToHistory(data):
-            # set message body to empty string by default, and later
-            # also forward message without body (chat state notification...)
             try:
-                _delay = delay.Delay.fromElement(filter(lambda elm: elm.name == 'delay', message.elements())[0])
-                timestamp = timegm(_delay.stamp.utctimetuple())
-                data['extra']['archive'] = str(timestamp)
+                timestamp = data['extra']['timestamp']  # timestamp added by XEP-0203
+            except KeyError:
+                self.host.memory.addToHistory(jid.JID(data['from']), jid.JID(data['to']), data['body'], data['type'], data['extra'], profile=self.parent.profile)
+            else:
                 if data['type'] != 'groupchat':  # XXX: we don't save delayed messages in history for groupchats
                     #TODO: add delayed messages to history if they aren't already in it
+                    data['extra']['archive'] = timestamp  # FIXME: this "archive" is actually never used
                     self.host.memory.addToHistory(jid.JID(data['from']), jid.JID(data['to']), data['body'], data['type'], data['extra'], timestamp, profile=self.parent.profile)
-            except IndexError:
-                self.host.memory.addToHistory(jid.JID(data['from']), jid.JID(data['to']), data['body'], data['type'], data['extra'], profile=self.parent.profile)
             return data
 
         def treatmentsEb(failure):
@@ -172,14 +190,44 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
     def __init__(self, host):
         xmppim.RosterClientProtocol.__init__(self)
         self.host = host
-        self.got_roster = defer.Deferred()
+        self.got_roster = defer.Deferred() # called when roster is received and ready
         #XXX: the two following dicts keep a local copy of the roster
-        self._groups = {}  # map from groups to bare jids: key=group value=set of bare jids
-        self._jids = {}  # map from bare jids to RosterItem: key=jid value=RosterItem
+        self._groups = {}  # map from groups to jids: key=group value=set of jids
+        self._jids = None  # map from jids to RosterItem: key=jid value=RosterItem
 
     def rosterCb(self, roster):
-        for raw_jid, item in roster.iteritems():
-            self.onRosterSet(item)
+        assert roster is not None # FIXME: must be managed with roster versioning
+        self._groups.clear()
+        self._jids = roster
+        for item in roster.itervalues():
+            if not item.subscriptionTo and not item.subscriptionFrom and not item.ask:
+                #XXX: current behaviour: we don't want contact in our roster list
+                # if there is no presence subscription
+                # may change in the future
+                log.info(u"Removing contact {} from roster because there is no presence subscription".format(item.jid))
+                self.removeItem(item.entity) # FIXME: to be checked
+            else:
+                self._registerItem(item)
+
+    def _registerItem(self, item):
+        """Register item in local cache
+
+        item must be already registered in self._jids before this method is called
+        @param item (RosterIem): item added
+        """
+        log.debug(u"registering item: {}".format(item.entity.full()))
+        if item.entity.resource:
+            log.warning(u"Received a roster item with a resource, this is not common but not restricted by RFC 6121, this case may be not well tested.")
+        if not item.subscriptionTo:
+            if not item.subscriptionFrom:
+                log.info(_(u"There's no subscription between you and [{}]!").format(item.entity.full()))
+            else:
+                log.info(_(u"You are not subscribed to [{}]!").format(item.entity.full()))
+        if not item.subscriptionFrom:
+            log.info(_(u"[{}] is not subscribed to you!").format(item.entity.full()))
+
+        for group in item.groups:
+            self._groups.setdefault(group, set()).add(item.entity)
 
     def requestRoster(self):
         """ ask the server for Roster list """
@@ -190,31 +238,9 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
     def removeItem(self, to_jid):
         """Remove a contact from roster list
         @param to_jid: a JID instance
+        @return: Deferred
         """
-        xmppim.RosterClientProtocol.removeItem(self, to_jid)
-        #TODO: check IQ result
-
-    #XXX: disabled (cf http://wokkel.ik.nu/ticket/56))
-    #def addItem(self, to):
-        #"""Add a contact to roster list"""
-        #xmppim.RosterClientProtocol.addItem(self, to)
-        #TODO: check IQ result"""
-
-    def updateItem(self, roster_item):
-        """
-        Update an item of the contact list.
-
-        @param roster_item: item to update
-        """
-        iq = compat.IQ(self.xmlstream, 'set')
-        iq.addElement((xmppim.NS_ROSTER, 'query'))
-        item = iq.query.addElement('item')
-        item['jid'] = roster_item.jid.userhost()
-        if roster_item.name:
-            item['name'] = roster_item.name
-        for group in roster_item.groups:
-            item.addElement('group', content=group)
-        return iq.send()
+        return xmppim.RosterClientProtocol.removeItem(self, to_jid)
 
     def getAttributes(self, item):
         """Return dictionary of attributes as used in bridge from a RosterItem
@@ -228,69 +254,64 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
             item_attr['name'] = item.name
         return item_attr
 
-    def onRosterSet(self, item):
-        """Called when a new/update roster item is received"""
-        #TODO: send a signal to frontends
-        if not item.subscriptionTo and not item.subscriptionFrom and not item.ask:
-            #XXX: current behaviour: we don't want contact in our roster list
-            # if there is no presence subscription
-            # may change in the future
-            self.removeItem(item.jid)
-            return
-        log.info(_("new contact in roster list: %s") % item.jid.full())
-        if not item.subscriptionTo:
-            log.warning(_("You are not subscribed to this contact !"))
-        if not item.subscriptionFrom:
-            log.warning(_("This contact is not subscribed to you !"))
-        #self.host.memory.addContact(item.jid, item_attr, item.groups, self.parent.profile)
+    def setReceived(self, request):
+        #TODO: implement roster versioning (cf RFC 6121 §2.6)
+        item = request.item
+        try:  # update the cache for the groups the contact has been removed from
+            left_groups = set(self._jids[item.entity].groups).difference(item.groups)
+            for group in left_groups:
+                jids_set = self._groups[group]
+                jids_set.remove(item.entity)
+                if not jids_set:
+                    del self._groups[group]
+        except KeyError:
+            pass  # no previous item registration (or it's been cleared)
+        self._jids[item.entity] = item
+        self._registerItem(item)
+        self.host.bridge.newContact(item.entity.full(), self.getAttributes(item), item.groups, self.parent.profile)
 
-        bare_jid = item.jid.userhostJID()
-        self._jids[bare_jid] = item
-        for group in item.groups:
-            self._groups.setdefault(group, set()).add(bare_jid)
-        self.host.bridge.newContact(item.jid.full(), self.getAttributes(item), item.groups, self.parent.profile)
-
-    def onRosterRemove(self, entity):
-        """Called when a roster removal event is received"""
-        print _("removing %s from roster list") % entity.full()
-        bare_jid = entity.userhostJID()
+    def removeReceived(self, request):
+        entity = request.item.entity
+        log.info(u"removing %s from roster list" % entity.full())
 
         # we first remove item from local cache (self._groups and self._jids)
         try:
-            item = self._jids.pop(bare_jid)
+            item = self._jids.pop(entity)
         except KeyError:
-            log.warning("Received a roster remove event for an item not in cache")
+            log.error(u"Received a roster remove event for an item not in cache ({})".format(entity))
             return
         for group in item.groups:
             try:
                 jids_set = self._groups[group]
-                jids_set.remove(bare_jid)
+                jids_set.remove(entity)
                 if not jids_set:
                     del self._groups[group]
             except KeyError:
-                log.warning("there is not cache for the group [%(groups)s] of the removed roster item [%(jid)s]" %
-                        {"group": group, "jid": bare_jid})
+                log.warning(u"there is no cache for the group [%(group)s] of the removed roster item [%(jid)s]" %
+                            {"group": group, "jid": entity})
 
         # then we send the bridge signal
-        self.host.bridge.contactDeleted(entity.userhost(), self.parent.profile)
+        self.host.bridge.contactDeleted(entity.full(), self.parent.profile)
 
     def getGroups(self):
         """Return a list of groups"""
         return self._groups.keys()
 
-    def getItem(self, jid):
+    def getItem(self, entity_jid):
         """Return RosterItem for a given jid
-        @param jid: jid of the contact
-        @return: RosterItem or None if contact is not in cache"""
-        return self._jids.get(jid.userhostJID(), None)
 
-    def getBareJids(self):
-        """Return all bare jids (as unicode) of the roster"""
+        @param entity_jid: jid of the contact
+        @return: RosterItem or None if contact is not in cache
+        """
+        return self._jids.get(entity_jid, None)
+
+    def getJids(self):
+        """Return all jids of the roster"""
         return self._jids.keys()
 
     def isJidInRoster(self, entity_jid):
         """Return True if jid is in roster"""
-        return entity_jid.userhostJID() in self._jids
+        return entity_jid in self._jids
 
     def getItems(self):
         """Return all items of the roster"""
@@ -300,7 +321,29 @@ class SatRosterProtocol(xmppim.RosterClientProtocol):
         try:
             return self._groups[group]
         except KeyError:
-            raise exceptions.UnknownGroupError
+            raise exceptions.UnknownGroupError(group)
+
+    def getJidsSet(self, type_, groups=None):
+        """Helper method to get a set of jids
+
+        @param type_(unicode): one of:
+            C.ALL: get all jids from roster
+            C.GROUP: get jids from groups (listed in "groups")
+        @groups(list[unicode]): list of groups used if type_==C.GROUP
+        @return (set(jid.JID)): set of selected jids
+        """
+        if type_ == C.ALL and groups is not None:
+            raise ValueError('groups must not be set for {} type'.format(C.ALL))
+
+        if type_ == C.ALL:
+            return set(self.getJids())
+        elif type_ == C.GROUP:
+            jids = set()
+            for group in groups:
+                jids.update(self.getJidsFromGroup(group))
+            return jids
+        else:
+            raise ValueError(u'Unexpected type_ {}'.format(type_))
 
 
 class SatPresenceProtocol(xmppim.PresenceClientProtocol):
@@ -315,46 +358,59 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
         super(SatPresenceProtocol, self).send(obj)
 
     def availableReceived(self, entity, show=None, statuses=None, priority=0):
-        log.debug(_("presence update for [%(entity)s] (available, show=%(show)s statuses=%(statuses)s priority=%(priority)d)") % {'entity': entity, 'show': show, 'statuses': statuses, 'priority': priority})
+        log.debug(_(u"presence update for [%(entity)s] (available, show=%(show)s statuses=%(statuses)s priority=%(priority)d)") % {'entity': entity, C.PRESENCE_SHOW: show, C.PRESENCE_STATUSES: statuses, C.PRESENCE_PRIORITY: priority})
 
         if not statuses:
             statuses = {}
 
         if None in statuses:  # we only want string keys
-            statuses["default"] = statuses[None]
-            del statuses[None]
+            statuses[C.PRESENCE_STATUSES_DEFAULT] = statuses.pop(None)
+
+        if not self.host.trigger.point("presenceReceived", entity, show, priority, statuses, self.parent.profile):
+            return
 
         self.host.memory.setPresenceStatus(entity, show or "",
                                            int(priority), statuses,
                                            self.parent.profile)
 
-        # uncomment these two lines if you need the trigger
-        #if not self.host.trigger.point("presenceReceived", entity, "unavailable", 0, statuses, self.parent.profile):
-        #    return
-
         # now it's time to notify frontends
         self.host.bridge.presenceUpdate(entity.full(), show or "",
                                         int(priority), statuses,
                                         self.parent.profile)
 
     def unavailableReceived(self, entity, statuses=None):
-        log.debug(_("presence update for [%(entity)s] (unavailable, statuses=%(statuses)s)") % {'entity': entity, 'statuses': statuses})
+        log.debug(_(u"presence update for [%(entity)s] (unavailable, statuses=%(statuses)s)") % {'entity': entity, C.PRESENCE_STATUSES: statuses})
 
         if not statuses:
             statuses = {}
 
         if None in statuses:  # we only want string keys
-            statuses["default"] = statuses[None]
-            del statuses[None]
-        self.host.memory.setPresenceStatus(entity, "unavailable", 0, statuses, self.parent.profile)
+            statuses[C.PRESENCE_STATUSES_DEFAULT] = statuses.pop(None)
 
         if not self.host.trigger.point("presenceReceived", entity, "unavailable", 0, statuses, self.parent.profile):
             return
 
         # now it's time to notify frontends
-        self.host.bridge.presenceUpdate(entity.full(), "unavailable", 0, statuses, self.parent.profile)
+        # if the entity is not known yet in this session or is already unavailable, there is no need to send an unavailable signal
+        try:
+            presence = self.host.memory.getEntityDatum(entity, "presence", self.parent.profile)
+        except (KeyError, exceptions.UnknownEntityError):
+            # the entity has not been seen yet in this session
+            pass
+        else:
+            if presence.show != C.PRESENCE_UNAVAILABLE:
+                self.host.bridge.presenceUpdate(entity.full(), C.PRESENCE_UNAVAILABLE, 0, statuses, self.parent.profile)
+
+        self.host.memory.setPresenceStatus(entity, C.PRESENCE_UNAVAILABLE, 0, statuses, self.parent.profile)
 
     def available(self, entity=None, show=None, statuses=None, priority=None):
+        """Set a presence and statuses.
+
+        @param entity (jid.JID): entity
+        @param show (unicode): value in ('unavailable', '', 'away', 'xa', 'chat', 'dnd')
+        @param statuses (dict{unicode: unicode}): multilingual statuses with
+            the entry key beeing a language code on 2 characters or "default".
+        """
         if priority is None:
             try:
                 priority = int(self.host.memory.getParamA("Priority", "Connection", profile_key=self.parent.profile))
@@ -366,16 +422,14 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
 
         # default for us is None for wokkel
         # so we must temporarily switch to wokkel's convention...
-        if 'default' in statuses:
-            statuses[None] = statuses['default']
-            del statuses['default']
+        if C.PRESENCE_STATUSES_DEFAULT in statuses:
+            statuses[None] = statuses.pop(C.PRESENCE_STATUSES_DEFAULT)
 
         presence_elt = xmppim.AvailablePresence(entity, show, statuses, priority)
 
         # ... before switching back
         if None in statuses:
-            statuses['default'] = statuses[None]
-            del statuses[None]
+            statuses['default'] = statuses.pop(None)
 
         if not self.host.trigger.point("presence_available", presence_elt, self.parent):
             return
@@ -394,15 +448,15 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
         self.host.memory.delWaitingSub(entity.userhost(), self.parent.profile)
 
     def subscribedReceived(self, entity):
-        log.debug(_("subscription approved for [%s]") % entity.userhost())
+        log.debug(_(u"subscription approved for [%s]") % entity.userhost())
         self.host.bridge.subscribe('subscribed', entity.userhost(), self.parent.profile)
 
     def unsubscribedReceived(self, entity):
-        log.debug(_("unsubscription confirmed for [%s]") % entity.userhost())
+        log.debug(_(u"unsubscription confirmed for [%s]") % entity.userhost())
         self.host.bridge.subscribe('unsubscribed', entity.userhost(), self.parent.profile)
 
     def subscribeReceived(self, entity):
-        log.debug(_("subscription request from [%s]") % entity.userhost())
+        log.debug(_(u"subscription request from [%s]") % entity.userhost())
         item = self.parent.roster.getItem(entity)
         if item and item.subscriptionTo:
             # We automatically accept subscription if we are already subscribed to contact presence
@@ -413,7 +467,7 @@ class SatPresenceProtocol(xmppim.PresenceClientProtocol):
             self.host.bridge.subscribe('subscribe', entity.userhost(), self.parent.profile)
 
     def unsubscribeReceived(self, entity):
-        log.debug(_("unsubscription asked for [%s]") % entity.userhost())
+        log.debug(_(u"unsubscription asked for [%s]") % entity.userhost())
         item = self.parent.roster.getItem(entity)
         if item and item.subscriptionFrom:  # we automatically remove contact
             log.debug(_('automatic contact deletion'))
@@ -448,14 +502,14 @@ class RegisteringAuthenticator(xmlstream.ConnectAuthenticator):
         self.user_email = email
         self.deferred = deferred
         self.profile = profile
-        log.debug(_("Registration asked for %(user)s@%(host)s") % {'user': user_login, 'host': jabber_host})
+        log.debug(_(u"Registration asked for %(user)s@%(host)s") % {'user': user_login, 'host': jabber_host})
 
     def connectionMade(self):
-        log.debug(_("Connection made with %s" % self.jabber_host))
+        log.debug(_(u"Connection made with %s" % self.jabber_host))
         self.xmlstream.namespace = "jabber:client"
         self.xmlstream.sendHeader()
 
-        iq = compat.IQ(self.xmlstream, 'set')
+        iq = xmlstream.IQ(self.xmlstream, 'set')
         iq["to"] = self.jabber_host
         query = iq.addElement(('jabber:iq:register', 'query'))
         _user = query.addElement('username')
@@ -469,11 +523,11 @@ class RegisteringAuthenticator(xmlstream.ConnectAuthenticator):
         d.chainDeferred(self.deferred)
 
     def registrationAnswer(self, answer):
-        log.debug(_("Registration answer: %s") % answer.toXml())
+        log.debug(_(u"Registration answer: %s") % answer.toXml())
         self.xmlstream.sendFooter()
 
     def registrationFailure(self, failure):
-        log.info(_("Registration failure: %s") % str(failure.value))
+        log.info(_("Registration failure: %s") % unicode(failure.value))
         self.xmlstream.sendFooter()
         raise failure.value
 
