@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT: a jabber client
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -114,7 +114,7 @@ class SqliteStorage(object):
 
             if statements is None:
                 return defer.succeed(None)
-            log.debug("===== COMMITING STATEMENTS =====\n%s\n============\n\n" % '\n'.join(statements))
+            log.debug(u"===== COMMITING STATEMENTS =====\n%s\n============\n\n" % '\n'.join(statements))
             d = self.dbpool.runInteraction(self._updateDb, tuple(statements))
             return d
 
@@ -170,13 +170,20 @@ class SqliteStorage(object):
         @param name: name of the profile
         @return: deferred triggered once profile is actually deleted"""
         def deletionError(failure):
-            log.error(_("Can't delete profile [%s]") % name)
+            log.error(_(u"Can't delete profile [%s]") % name)
             return failure
 
         def delete(txn):
-            del self.profiles[name]
+            profile_id = self.profiles.pop(name)
             txn.execute("PRAGMA foreign_keys = ON")
             txn.execute("DELETE FROM profiles WHERE name = ?", (name,))
+            # FIXME: the following queries should be done by the ON DELETE CASCADE
+            #        but it seems they are not, so we explicitly do them by security
+            #        this need more investigation
+            txn.execute("DELETE FROM history WHERE profile_id = ?", (profile_id,))
+            txn.execute("DELETE FROM param_ind WHERE profile_id = ?", (profile_id,))
+            txn.execute("DELETE FROM private_ind WHERE profile_id = ?", (profile_id,))
+            txn.execute("DELETE FROM private_ind_bin WHERE profile_id = ?", (profile_id,))
             return None
 
         d = self.dbpool.runInteraction(delete)
@@ -194,7 +201,7 @@ class SqliteStorage(object):
             for param in result:
                 category, name, value = param
                 params_gen[(category, name)] = value
-        log.debug(_("loading general parameters from database"))
+        log.debug(_(u"loading general parameters from database"))
         return self.dbpool.runQuery("SELECT category,name,value FROM param_gen").addCallback(fillParams)
 
     def loadIndParams(self, params_ind, profile):
@@ -207,7 +214,7 @@ class SqliteStorage(object):
             for param in result:
                 category, name, value = param
                 params_ind[(category, name)] = value
-        log.debug(_("loading individual parameters from database"))
+        log.debug(_(u"loading individual parameters from database"))
         d = self.dbpool.runQuery("SELECT category,name,value FROM param_ind WHERE profile_id=?", (self.profiles[profile], ))
         d.addCallback(fillParams)
         return d
@@ -229,7 +236,7 @@ class SqliteStorage(object):
         @param value: value to set
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO param_gen(category,name,value) VALUES (?,?,?)", (category, name, value))
-        d.addErrback(lambda ignore: log.error(_("Can't set general parameter (%(category)s/%(name)s) in database" % {"category": category, "name": name})))
+        d.addErrback(lambda ignore: log.error(_(u"Can't set general parameter (%(category)s/%(name)s) in database" % {"category": category, "name": name})))
         return d
 
     def setIndParam(self, category, name, value, profile):
@@ -240,7 +247,7 @@ class SqliteStorage(object):
         @param profile: a profile which *must* exist
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO param_ind(category,name,profile_id,value) VALUES (?,?,?,?)", (category, name, self.profiles[profile], value))
-        d.addErrback(lambda ignore: log.error(_("Can't set individual parameter (%(category)s/%(name)s) for [%(profile)s] in database" % {"category": category, "name": name, "profile": profile})))
+        d.addErrback(lambda ignore: log.error(_(u"Can't set individual parameter (%(category)s/%(name)s) for [%(profile)s] in database" % {"category": category, "name": name, "profile": profile})))
         return d
 
     #History
@@ -260,17 +267,25 @@ class SqliteStorage(object):
         d = self.dbpool.runQuery("INSERT INTO history(source, source_res, dest, dest_res, timestamp, message, type, extra, profile_id) VALUES (?,?,?,?,?,?,?,?,?)",
                                  (from_jid.userhost(), from_jid.resource, to_jid.userhost(), to_jid.resource, timestamp or time(),
                                   message, _type, extra_, self.profiles[profile]))
-        d.addErrback(lambda ignore: log.error(_("Can't save following message in history: from [%(from_jid)s] to [%(to_jid)s] ==> [%(message)s]" %
+        d.addErrback(lambda ignore: log.error(_(u"Can't save following message in history: from [%(from_jid)s] to [%(to_jid)s] ==> [%(message)s]" %
                                           {"from_jid": from_jid.full(), "to_jid": to_jid.full(), "message": message})))
         return d
 
-    def getHistory(self, from_jid, to_jid, limit=0, between=True, profile=None):
-        """Store a new message in history
-        @param from_jid: source JID (full, or bare for catchall
-        @param to_jid: dest JID (full, or bare for catchall
-        @param size: maximum number of messages to get, or 0 for unlimited
+    def getHistory(self, from_jid, to_jid, limit=None, between=True, search=None, profile=None):
+        """Retrieve messages in history
+        @param from_jid (JID): source JID (full, or bare for catchall)
+        @param to_jid (JID): dest JID (full, or bare for catchall)
+        @param limit (int): maximum number of messages to get:
+            - 0 for no message (returns the empty list)
+            - None for unlimited
+        @param between (bool): confound source and dest (ignore the direction)
+        @param search (str): pattern to filter the history results
+        @param profile (str): %(doc_profile)s
+        @return: list of tuple as in http://wiki.goffi.org/wiki/Bridge_API#getHistory
         """
         assert(profile)
+        if limit == 0:
+            return defer.succeed([])
 
         def sqliteToList(query_result):
             query_result.reverse()
@@ -304,10 +319,14 @@ class SqliteStorage(object):
         else:
             query_parts.append("%s AND %s" % (test_jid('source', from_jid),
                                               test_jid('dest', to_jid)))
+        if search:
+            # TODO: use REGEXP (function to be defined) instead of GLOB: https://www.sqlite.org/lang_expr.html
+            query_parts.append("AND message GLOB ?")
+            values.append("*%s*" % search)
 
         query_parts.append("ORDER BY timestamp DESC")
 
-        if limit:
+        if limit is not None:
             query_parts.append("LIMIT ?")
             values.append(limit)
 
@@ -325,9 +344,9 @@ class SqliteStorage(object):
             for private in result:
                 key, value = private
                 private_gen[key] = value
-        log.debug(_("loading general private values [namespace: %s] from database") % (namespace, ))
+        log.debug(_(u"loading general private values [namespace: %s] from database") % (namespace,))
         d = self.dbpool.runQuery("SELECT key,value FROM private_gen WHERE namespace=?", (namespace, )).addCallback(fillPrivates)
-        return d.addErrback(lambda x: log.debug(_("No data present in database for namespace %s") % namespace))
+        return d.addErrback(lambda x: log.debug(_(u"No data present in database for namespace %s") % namespace))
 
     def loadIndPrivates(self, private_ind, namespace, profile):
         """Load individual private values
@@ -340,10 +359,10 @@ class SqliteStorage(object):
             for private in result:
                 key, value = private
                 private_ind[key] = value
-        log.debug(_("loading individual private values [namespace: %s] from database") % (namespace, ))
+        log.debug(_(u"loading individual private values [namespace: %s] from database") % (namespace,))
         d = self.dbpool.runQuery("SELECT key,value FROM private_ind WHERE namespace=? AND profile_id=?", (namespace, self.profiles[profile]))
         d.addCallback(fillPrivates)
-        return d.addErrback(lambda x: log.debug(_("No data present in database for namespace %s") % namespace))
+        return d.addErrback(lambda x: log.debug(_(u"No data present in database for namespace %s") % namespace))
 
     def setGenPrivate(self, namespace, key, value):
         """Save the general private value in database
@@ -352,7 +371,7 @@ class SqliteStorage(object):
         @param value: value to set
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO private_gen(namespace,key,value) VALUES (?,?,?)", (namespace, key, value))
-        d.addErrback(lambda ignore: log.error(_("Can't set general private value (%(key)s) [namespace:%(namespace)s] in database" %
+        d.addErrback(lambda ignore: log.error(_(u"Can't set general private value (%(key)s) [namespace:%(namespace)s] in database" %
                      {"namespace": namespace, "key": key})))
         return d
 
@@ -364,7 +383,7 @@ class SqliteStorage(object):
         @param profile: a profile which *must* exist
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO private_ind(namespace,key,profile_id,value) VALUES (?,?,?,?)", (namespace, key, self.profiles[profile], value))
-        d.addErrback(lambda ignore: log.error(_("Can't set individual private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
+        d.addErrback(lambda ignore: log.error(_(u"Can't set individual private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
                      {"namespace": namespace, "key": key, "profile": profile})))
         return d
 
@@ -374,7 +393,7 @@ class SqliteStorage(object):
         @param key: key of the private value
         @return: deferred"""
         d = self.dbpool.runQuery("DELETE FROM private_gen WHERE namespace=? AND key=?", (namespace, key))
-        d.addErrback(lambda ignore: log.error(_("Can't delete general private value (%(key)s) [namespace:%(namespace)s] in database" %
+        d.addErrback(lambda ignore: log.error(_(u"Can't delete general private value (%(key)s) [namespace:%(namespace)s] in database" %
                      {"namespace": namespace, "key": key})))
         return d
 
@@ -385,7 +404,7 @@ class SqliteStorage(object):
         @param profile: a profile which *must* exist
         @return: deferred"""
         d = self.dbpool.runQuery("DELETE FROM private_ind WHERE namespace=? AND key=? AND profile=?)", (namespace, key, self.profiles[profile]))
-        d.addErrback(lambda ignore: log.error(_("Can't delete individual private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
+        d.addErrback(lambda ignore: log.error(_(u"Can't delete individual private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
                      {"namespace": namespace, "key": key, "profile": profile})))
         return d
 
@@ -399,9 +418,9 @@ class SqliteStorage(object):
             for private in result:
                 key, value = private
                 private_gen[key] = pickle.loads(str(value))
-        log.debug(_("loading general private binary values [namespace: %s] from database") % (namespace, ))
+        log.debug(_(u"loading general private binary values [namespace: %s] from database") % (namespace,))
         d = self.dbpool.runQuery("SELECT key,value FROM private_gen_bin WHERE namespace=?", (namespace, )).addCallback(fillPrivates)
-        return d.addErrback(lambda x: log.debug(_("No binary data present in database for namespace %s") % namespace))
+        return d.addErrback(lambda x: log.debug(_(u"No binary data present in database for namespace %s") % namespace))
 
     def loadIndPrivatesBinary(self, private_ind, namespace, profile):
         """Load individual private binary values
@@ -414,10 +433,10 @@ class SqliteStorage(object):
             for private in result:
                 key, value = private
                 private_ind[key] = pickle.loads(str(value))
-        log.debug(_("loading individual private binary values [namespace: %s] from database") % (namespace, ))
+        log.debug(_(u"loading individual private binary values [namespace: %s] from database") % (namespace,))
         d = self.dbpool.runQuery("SELECT key,value FROM private_ind_bin WHERE namespace=? AND profile_id=?", (namespace, self.profiles[profile]))
         d.addCallback(fillPrivates)
-        return d.addErrback(lambda x: log.debug(_("No binary data present in database for namespace %s") % namespace))
+        return d.addErrback(lambda x: log.debug(_(u"No binary data present in database for namespace %s") % namespace))
 
     def setGenPrivateBinary(self, namespace, key, value):
         """Save the general private binary value in database
@@ -426,7 +445,7 @@ class SqliteStorage(object):
         @param value: value to set
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO private_gen_bin(namespace,key,value) VALUES (?,?,?)", (namespace, key, pickle.dumps(value, 0)))
-        d.addErrback(lambda ignore: log.error(_("Can't set general private binary value (%(key)s) [namespace:%(namespace)s] in database" %
+        d.addErrback(lambda ignore: log.error(_(u"Can't set general private binary value (%(key)s) [namespace:%(namespace)s] in database" %
                      {"namespace": namespace, "key": key})))
         return d
 
@@ -438,7 +457,7 @@ class SqliteStorage(object):
         @param profile: a profile which *must* exist
         @return: deferred"""
         d = self.dbpool.runQuery("REPLACE INTO private_ind_bin(namespace,key,profile_id,value) VALUES (?,?,?,?)", (namespace, key, self.profiles[profile], pickle.dumps(value, 0)))
-        d.addErrback(lambda ignore: log.error(_("Can't set individual binary private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
+        d.addErrback(lambda ignore: log.error(_(u"Can't set individual binary private value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
                      {"namespace": namespace, "key": key, "profile": profile})))
         return d
 
@@ -448,7 +467,7 @@ class SqliteStorage(object):
         @param key: key of the private value
         @return: deferred"""
         d = self.dbpool.runQuery("DELETE FROM private_gen_bin WHERE namespace=? AND key=?", (namespace, key))
-        d.addErrback(lambda ignore: log.error(_("Can't delete general private binary value (%(key)s) [namespace:%(namespace)s] in database" %
+        d.addErrback(lambda ignore: log.error(_(u"Can't delete general private binary value (%(key)s) [namespace:%(namespace)s] in database" %
                      {"namespace": namespace, "key": key})))
         return d
 
@@ -459,7 +478,7 @@ class SqliteStorage(object):
         @param profile: a profile which *must* exist
         @return: deferred"""
         d = self.dbpool.runQuery("DELETE FROM private_ind_bin WHERE namespace=? AND key=? AND profile=?)", (namespace, key, self.profiles[profile]))
-        d.addErrback(lambda ignore: log.error(_("Can't delete individual private binary value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
+        d.addErrback(lambda ignore: log.error(_(u"Can't delete individual private binary value (%(key)s) [namespace: %(namespace)s] for [%(profile)s] in database" %
                      {"namespace": namespace, "key": key, "profile": profile})))
         return d
     ##Helper methods##
@@ -709,7 +728,7 @@ class Updater(object):
     @defer.inlineCallbacks
     def update2raw(self, update, dev_version=False):
         """ Transform update data to raw SQLite statements
-        @param upadte: update data as returned by generateUpdateData
+        @param update: update data as returned by generateUpdateData
         @param dev_version: if True, update will be done in dev mode: no deletion will be done, instead a message will be shown. This prevent accidental lost of data while working on the code/database.
         @return: list of string with SQL statements needed to update the base
 
@@ -769,7 +788,7 @@ class Updater(object):
                 try:
                     id_ = result[0][0]
                 except IndexError:
-                    log.error("Profile of id %d is referenced in 'param_ind' but it doesn't exist!" % profile_id)
+                    log.error(u"Profile of id %d is referenced in 'param_ind' but it doesn't exist!" % profile_id)
                     return defer.succeed(None)
 
                 sat_password = xmpp_password
