@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin for adding D-Bus to Ad-Hoc Commands
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -17,15 +17,18 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from sat.core.i18n import _, D_
+from sat.core.i18n import _
 from sat.core.constants import Const as C
 from sat.core.log import getLogger
 log = getLogger(__name__)
-from twisted.words.protocols.jabber import jid
-from twisted.internet import defer, reactor
+from sat.core import exceptions
+from twisted.internet import defer
 from wokkel import data_form
-from lxml import etree
-from os import path
+try:
+    from lxml import etree
+except ImportError:
+    raise exceptions.MissingModule(u"Missing module lxml, please download/install it from http://lxml.de/")
+import os.path
 import uuid
 import dbus
 from dbus.mainloop.glib import DBusGMainLoop
@@ -103,7 +106,7 @@ class AdHocDBus(object):
         el = etree.fromstring(introspect_xml)
         for node in el.iterchildren('node', 'interface'):
             if node.tag == 'node':
-                new_path = path.join(proxy.object_path, node.get('name'))
+                new_path = os.path.join(proxy.object_path, node.get('name'))
                 new_proxy = self.session_bus.get_object(bus_name, new_path, introspect=False)
                 yield self._introspect(methods, bus_name, new_proxy)
             elif node.tag == 'interface':
@@ -127,7 +130,7 @@ class AdHocDBus(object):
         bus_names = [bus_name for bus_name in bus_names if '.' + prog_name in bus_name]
         if not bus_names:
             log.info("Can't find any bus for [%s]" % prog_name)
-            return
+            defer.returnValue(("", []))
         bus_names.sort()
         for bus_name in bus_names:
             if bus_name.endswith(prog_name):
@@ -183,7 +186,7 @@ class AdHocDBus(object):
                     x_elt = command_elt.elements(data_form.NS_X_DATA,'x').next()
                     answer_form = data_form.Form.fromElement(x_elt)
                     command = answer_form['command']
-                except KeyError, StopIteration:
+                except (KeyError, StopIteration):
                     raise self.XEP_0050.AdHocError(self.XEP_0050.ERROR.BAD_PAYLOAD)
 
                 if command not in names_map:
@@ -192,7 +195,7 @@ class AdHocDBus(object):
                 path, iface, command = names_map[command]
                 proxy = self.session_bus.get_object(bus_name, path)
 
-                d = self._DBusAsyncCall(proxy, command, interface=iface)
+                self._DBusAsyncCall(proxy, command, interface=iface)
 
                 # job done, we can end the session, except if we have FLAG_LOOP
                 if FLAG_LOOP in flags:
