@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin to export commands (experimental)
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -23,7 +23,7 @@ log = getLogger(__name__)
 from twisted.words.protocols.jabber import jid
 from twisted.internet import reactor, protocol
 
-from sat.tools.misc import SkipOtherTriggers
+from sat.tools import trigger
 from sat.tools.utils import clean_ustr
 
 PLUGIN_INFO = {
@@ -57,10 +57,10 @@ class ExportCommandProtocol(protocol.ProcessProtocol):
         log.info("connectionMade :)")
 
     def outReceived(self, data):
-        self.parent.host.sendMessage(self.target, self._clean(data), no_trigger=True, profile_key=self.profile) 
+        self.parent.host.sendMessage(self.target, self._clean(data), no_trigger=True, profile_key=self.profile)
 
     def errReceived(self, data):
-        self.parent.host.sendMessage(self.target, self._clean(data), no_trigger=True, profile_key=self.profile) 
+        self.parent.host.sendMessage(self.target, self._clean(data), no_trigger=True, profile_key=self.profile)
 
     def processEnded(self, reason):
         log.info (u"process finished: %d" % (reason.value.exitCode,))
@@ -110,7 +110,7 @@ class CommandExport(object):
             body = [e for e in message.elements() if e.name == 'body'][0]
         except IndexError:
             # do not block message without body (chat state notification...)
-            log.warning("No body element found in message, following normal behaviour")
+            log.debug("No body element found in message, following normal behaviour")
             return True
 
         mess_data = unicode(body) + '\n'
@@ -124,7 +124,7 @@ class CommandExport(object):
                 _continue &= process.boolOption("continue")
                 exclusive |= process.boolOption("exclusive")
             if exclusive:
-                raise SkipOtherTriggers
+                raise trigger.SkipOtherTriggers
             return _continue
 
         return True
@@ -142,16 +142,16 @@ class CommandExport(object):
         """
         profile = self.host.memory.getProfileName(profile_key)
         if not profile:
-            log.warning("Unknown profile [%s]" % (profile,))
+            log.warning(u"Unknown profile [%s]" % (profile,))
             return
-        
+
         for target in targets:
             try:
                 _jid = jid.JID(target)
                 if not _jid.user or not _jid.host:
                     raise jid.InvalidFormat
                 _jid = _jid.userhostJID()
-            except (jid.InvalidFormat, RuntimeError):
+            except (RuntimeError, jid.InvalidFormat, AttributeError):
                 log.info(u"invalid target ignored: %s" % (target,))
                 continue
             process_prot = ExportCommandProtocol(self, _jid, options, profile)
