@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin for account creation (experimental)
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -25,7 +25,7 @@ from sat.tools import xml_tools
 from sat.memory.memory import Sessions
 from sat.memory.crypto import PasswordHasher
 from sat.core.constants import Const as C
-
+import ConfigParser
 from twisted.internet import reactor, defer, protocol
 from twisted.python.procutils import which
 from twisted.python.failure import Failure
@@ -33,6 +33,9 @@ from twisted.mail.smtp import sendmail
 from os.path import join, dirname
 from email.mime.text import MIMEText
 
+import random
+
+
 PLUGIN_INFO = {
     "name": "Account Plugin",
     "import_name": "MISC-ACCOUNT",
@@ -53,10 +56,16 @@ CONFIG_SECTION = "plugin account"
 
 default_conf = {"email_from": "NOREPLY@example.net",
                 "email_server": "localhost",
-                "admin_email": "admin@example.net",
+                "email_sender_domain": "",
+                "email_port": 25,
+                "email_username": "",
+                "email_password": "",
+                "email_starttls": "false",
+                "email_auth": "false",
+                "email_admins_list": [],
+                "admin_email": "",
                 "new_account_server": "localhost",
                 "new_account_domain": "example.net",
-                "new_account_resource": "libervia",
                 "prosody_path": None, # prosody path (where prosodyctl will be executed from), or None to automaticaly find it
                 "prosodyctl": "prosodyctl",
                 "reserved_list": ['libervia'] # profiles which can't be used
@@ -102,6 +111,7 @@ class ProsodyRegisterProtocol(protocol.ProcessProtocol):
     @classmethod
     def prosodyctl(cls, plugin, command, password=None, profile=None):
         """Create a new ProsodyRegisterProtocol and execute the given prosodyctl command.
+
         @param plugin: instance of MiscAccount
         @param command: the command to execute: "adduser", "passwd" or "deluser"
         @param password: the user new password (leave to None for "deluser" command)
@@ -127,15 +137,17 @@ class MiscAccount(object):
         host.bridge.addMethod("registerSatAccount", ".plugin", in_sign='sss', out_sign='', method=self._registerAccount, async=True)
         host.bridge.addMethod("getNewAccountDomain", ".plugin", in_sign='', out_sign='s', method=self.getNewAccountDomain, async=False)
         host.bridge.addMethod("getAccountDialogUI", ".plugin", in_sign='s', out_sign='s', method=self._getAccountDialogUI, async=False)
+        host.bridge.addMethod("asyncConnectWithXMPPCredentials", ".plugin", in_sign='ss', out_sign='b', method=self.asyncConnectWithXMPPCredentials, async=True)
         self._prosody_path = self.getConfig('prosody_path')
         if self._prosody_path is None:
             paths = which(self.getConfig('prosodyctl'))
             if not paths:
-                log.error(_("Can't find %s") % (self.getConfig('prosodyctl'), ))
+                log.error(_(u"Can't find %s") % (self.getConfig('prosodyctl'),))
             else:
                 self._prosody_path = dirname(paths[0])
-                log.info(_('Prosody path found: %s') % (self._prosody_path, ))
+                log.info(_(u'Prosody path found: %s') % (self._prosody_path,))
 
+        self.fixEmailAdmins()
         self._sessions = Sessions()
 
         self.__account_cb_id = host.registerCallback(self._accountDialogCb, with_data=True)
@@ -150,108 +162,201 @@ class MiscAccount(object):
 
         self.__delete_account_id = host.registerCallback(self.__deleteAccountCb, with_data=True)
 
+
+    # FIXME: remove this after some time, when the deprecated parameter is really abandoned
+    def fixEmailAdmins(self):
+        """Handle deprecated config option "admin_email" to fix the admin emails list"""
+        admin_email = self.getConfig('admin_email')
+        if not admin_email:
+            return
+        log.warning(u"admin_email parameter is deprecated, please use email_admins_list instead")
+        param_name = "email_admins_list"
+        try:
+            section = ""
+            value = self.host.memory.getConfig(section, param_name, Exception)
+        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+            section = CONFIG_SECTION
+            value = self.host.memory.getConfig(section, param_name, default_conf[param_name])
+
+        value = set(value)
+        value.add(admin_email)
+        self.host.memory.config.set(section, param_name, ",".join(value))
+
     def getConfig(self, name):
-        return self.host.memory.getConfig(CONFIG_SECTION, name) or default_conf[name]
+        if name.startswith("email_"):
+            # XXX: email_ parameters were first in [plugin account] section
+            #      but as it make more sense to have them in common with other plugins,
+            #      they can now be in [DEFAULT] section
+            try:
+                value = self.host.memory.getConfig(None, name, Exception)
+            except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
+                pass
+            else:
+                return value
+
+        return self.host.memory.getConfig(CONFIG_SECTION, name, default_conf[name])
+
+    def generatePassword(self):
+        """Generate a password with random characters.
+
+        @return unicode
+        """
+        random.seed()
+        #charset = [chr(i) for i in range(0x21,0x7F)] #XXX: this charset seems to have some issues with openfire
+        charset = [chr(i) for i in range(0x30,0x3A) + range(0x41,0x5B) + range (0x61,0x7B)]
+        return ''.join([random.choice(charset) for i in range(15)])
 
     def _registerAccount(self, email, password, profile):
+        return self.registerAccount(email, password, None, profile)
 
+    def registerAccount(self, email, password, jid_s, profile):
+        """Register a new profile, its associated XMPP account, send the confirmation emails.
+
+        @param email (unicode): where to send to confirmation email to
+        @param password (unicode): password chosen by the user
+            while be used for profile *and* XMPP account
+        @param jid_s (unicode): JID to re-use or to register:
+            - non empty value: bind this JID to the new sat profile
+            - None or "": register a new JID on the local XMPP server
+        @param profile
+        @return Deferred
         """
-        #Password Generation
-        #_charset = [chr(i) for i in range(0x21,0x7F)] #XXX: this charset seems to have some issues with openfire
-        _charset = [chr(i) for i in range(0x30,0x3A) + range(0x41,0x5B) + range (0x61,0x7B)]
-        import random
-        random.seed()
-        password = ''.join([random.choice(_charset) for i in range(15)])
+        d = self.createProfile(password, jid_s, profile)
+        d.addCallback(lambda dummy: self.sendEmails(email, profile))
+        return d
+
+    def createProfile(self, password, jid_s, profile):
+        """Register a new profile and its associated XMPP account.
+
+        @param password (unicode): password chosen by the user
+            while be used for profile *and* XMPP account
+        @param jid_s (unicode): JID to re-use or to register:
+            - non empty value: bind this JID to the new sat profile
+            - None or "": register a new JID on the local XMPP server
+        @param profile
+        @return Deferred
         """
-        if not email or not password or not profile:
+        if not password or not profile:
             raise exceptions.DataError
 
         if profile.lower() in self.getConfig('reserved_list'):
             return defer.fail(Failure(exceptions.ConflictError))
 
         d = self.host.memory.asyncCreateProfile(profile, password)
-        d.addCallback(self._profileRegistered, email, password, profile)
+        d.addCallback(lambda dummy: self.profileCreated(password, jid_s, profile))
         return d
 
-    def _profileRegistered(self, result, email, password, profile):
-        """Create the XMPP account and send the confirmation email.
+    def profileCreated(self, password, jid_s, profile):
+        """Create the XMPP account and set the profile connection parameters.
 
-        @param result: result of asyncCreateProfile
-        @param email: user email
-        @param password: chosen xmpp password
-        @param profile: %(doc_profile)s
-        @return: a deferred None value when all the processing is done
+        @param password (unicode): password chosen by the user
+        @param jid_s (unicode): JID to re-use or to register:
+            - non empty value: bind this JID to the new sat profile
+            - None or empty: register a new JID on the local XMPP server
+        @param profile
+        @return: Deferred
         """
-        # XXX: we use "prosodyctl adduser" because "register" doesn't check conflict
-        #      and just change the password if the account already exists
-        d = ProsodyRegisterProtocol.prosodyctl(self, 'adduser', password, profile)
-        d.addCallback(self._sendEmails, profile, email, password)
-        d.addCallback(lambda ignore: None)
+        # XXX: we use "prosodyctl adduser" because "register" doesn't check conflict and just change the password if the account already exists
+        if jid_s:
+            d = defer.succeed(None)
+        else:
+            d = ProsodyRegisterProtocol.prosodyctl(self, 'adduser', password, profile)
+            jid_s = "%s@%s" % (profile, self.getConfig('new_account_domain'))
 
         def setParams(dummy):
-            jid_s = "%s@%s/%s" % (profile, self.getConfig('new_account_domain'), self.getConfig('new_account_resource'))
             self.host.memory.setParam("JabberID", jid_s, "Connection", profile_key=profile)
             d = self.host.memory.setParam("Password", password, "Connection", profile_key=profile)
-            d.addCallback(lambda dummy: self.host.memory.auth_sessions.profileDelUnique(profile))
+            return d
 
         def removeProfile(failure):
             self.host.memory.asyncDeleteProfile(profile)
             return failure
 
+        d.addCallback(lambda dummy: self.host.memory.startSession(password, profile))
         d.addCallback(setParams)
+        d.addCallback(lambda dummy: self.host.memory.stopSession(profile))
         d.addErrback(removeProfile)
         return d
 
-    def _sendEmails(self, result, login, email, password):
-        #time to send the email
+    def sendEmails(self, email, profile):
+        # time to send the email
 
-        _email_host = self.getConfig('email_server')
-        _email_from = self.getConfig("email_from")
+        email_host = self.getConfig('email_server')
+        email_from = self.getConfig("email_from")
+        email_sender_domain = self.getConfig("email_sender_domain") or None
+        email_port = int(self.getConfig("email_port"))
+        email_username = self.getConfig("email_username") or None
+        email_password = self.getConfig("email_password") or None
+        email_starttls = C.bool(self.getConfig("email_starttls"))
+        email_auth = C.bool(self.getConfig("email_auth"))
+        domain = self.getConfig('new_account_domain')
 
-        def email_ok(ignore):
-            print ("Account creation email sent to %s" % email)
+        def sendEmail(recipients, msg):
+            return sendmail(email_host.encode("utf-8"),
+                            email_from.encode("utf-8"),
+                            [email.encode("utf-8") for email in recipients],
+                            msg.as_string(),
+                            senderDomainName=email_sender_domain.encode("utf-8") if email_sender_domain else None,
+                            port=email_port,
+                            username=email_username.encode("utf-8") if email_username else None,
+                            password=email_password.encode("utf-8") if email_password else None,
+                            requireAuthentication=email_starttls,
+                            requireTransportSecurity=email_auth)
 
-        def email_ko(ignore):
-            #TODO: return error code to user
-            log.error("Failed to send email to %s" % email)
+        # email to the administrators
+        admins_emails = self.getConfig('email_admins_list')
+        if not admins_emails:
+            log.warning(u"No known admin email, we can't send email to administrator(s).\nPlease fill email_admins_list parameter")
+            d_admin = defer.fail(exceptions.DataError("no admin email"))
+        else:
+            body = (u"""New account created: %(profile)s [%(email)s]""" % {'profile': profile, 'email': email}).encode('utf-8')
+            msg = MIMEText(body, 'plain', 'UTF-8')
+            msg['Subject'] = _('New Libervia account created')
+            msg['From'] = email_from
+            msg['To'] = ", ".join(admins_emails)
 
-        body = (u"""Welcome to Libervia, a Salut à Toi project part
+            d_admin = sendEmail(admins_emails, msg)
+        admins_emails_txt = u', '.join([u"<{}>".format(addr) for addr in admins_emails])
+        d_admin.addCallbacks(lambda dummy: log.debug(u"Account creation notification sent to admin(s) {}".format(admins_emails_txt)),
+                             lambda dummy: log.error(u"Failed to send account creation notification to admin {}".format(admins_emails_txt)))
+        if not email:
+            return d_admin
 
-/!\\ WARNING, THIS IS ONLY A TECHNICAL DEMO, DON'T USE THIS ACCOUNT FOR ANY SERIOUS PURPOSE /!\\
+        jid_s = self.host.memory.getParamA("JabberID", "Connection", profile_key=profile)
+        body = (_(u"""Welcome to Libervia, the web interface of Salut à Toi.
+
+Your account on {domain} has been successfully created. This is a demonstration version to show you the current status of the project. It is still under development, please keep it in mind!
 
 Here is your connection information:
----
-login: %(login)s
 
-Your Jabber ID (JID) is: %(jid)s
----
+Login on {domain}: {profile}
+Jabber ID (JID): {jid}
+Your password has been chosen by yourself during registration.
 
-To try with some contacts, you can use the directory: subscribe in your parameters, and use it with the Communication/Search directory menu.
+In the beginning, you have nobody to talk to. To find some contacts, you may use the users' directory:
+    - make yourself visible in "Service / Directory subscription".
+    - search for people with "Contacts" / Search directory".
 
-SàT website: http://www.salut-a-toi.org
-follow SàT news: http://www.goffi.org
+Any feedback welcome. Thank you!
 
-Any feedback welcome
-
-Cheers
-SàT team""" % {'login': login, 'jid': "%s@%s" % (login, self.getConfig('new_account_domain'))}).encode('utf-8')
+Salut à Toi association
+http://www.salut-a-toi.org
+""").format(profile=profile, jid=jid_s, domain=domain)).encode('utf-8')
         msg = MIMEText(body, 'plain', 'UTF-8')
-        msg['Subject'] = 'Libervia account created'
-        msg['From'] = _email_from
+        msg['Subject'] = _(u'Libervia account created')
+        msg['From'] = email_from
         msg['To'] = email
 
-        d_user = sendmail(_email_host, _email_from, email, msg.as_string())
-        d_user.addCallbacks(email_ok, email_ko)
+        def email_ko(dummy):
+            # TODO: return error code to user
+            log.error(u"Failed to send account creation confirmation to <%s>" % email)
 
-        #email to the administrator
-        body = (u"""New account created: %(login)s [%(email)s]""" % {'login': login, 'email': email}).encode('utf-8')
-        msg = MIMEText(body, 'plain', 'UTF-8')
-        msg['Subject'] = 'Libervia new account created'
-        msg['From'] = _email_from
-        msg['To'] = self.getConfig('admin_email')
-
-        d_admin = sendmail(_email_host, _email_from, self.getConfig('admin_email'), msg.as_string())
-        d_admin.addCallbacks(email_ok, email_ko)
+        # XXX: this will not fail when the email address doesn't exist
+        # FIXME: check email reception to validate email given by the user
+        # FIXME: delete the profile if the email could not been sent?
+        d_user = sendEmail([email], msg)
+        d_user.addCallbacks(lambda dummy: log.debug(u"Account creation confirmation sent to <%s>" % email),
+                            email_ko)
         return defer.DeferredList([d_user, d_admin])
 
     def getNewAccountDomain(self):
@@ -275,6 +380,8 @@ SàT team""" % {'login': login, 'jid': "%s@%s" % (login, self.getConfig('new_acc
         form_ui.addLabel(D_("New password (again)"))
         form_ui.addPassword("new_passwd2", value="")
 
+        # FIXME: uncomment and fix these features
+        """
         if 'GROUPBLOG' in self.host.plugins:
             tab_container.addTab("delete_posts", D_("Delete your posts"), container=xml_tools.PairsContainer)
             form_ui.addLabel(D_("Current profile password"))
@@ -289,6 +396,8 @@ SàT team""" % {'login': login, 'jid': "%s@%s" % (login, self.getConfig('new_acc
         form_ui.addPassword("delete_passwd", value="")
         form_ui.addLabel(D_("Delete your account"))
         form_ui.addBool("delete_checkbox", "false")
+        """
+
         return form_ui.toXml()
 
     @defer.inlineCallbacks
@@ -306,16 +415,20 @@ SàT team""" % {'login': login, 'jid': "%s@%s" % (login, self.getConfig('new_acc
 
         def error_ui(message=None):
             if not message:
-                D_("The provided profile password doesn't match.")
+                message = D_("The provided profile password doesn't match.")
             error_ui = xml_tools.XMLUI("popup", title=D_("Attempt failure"))
             error_ui.addText(message)
             return {'xmlui': error_ui.toXml()}
 
         # check for account deletion
+        # FIXME: uncomment and fix these features
+        """
         delete_passwd = data[xml_tools.SAT_FORM_PREFIX + 'delete_passwd']
         delete_checkbox = data[xml_tools.SAT_FORM_PREFIX + 'delete_checkbox']
         if delete_checkbox == 'true':
-            if verify(delete_passwd):
+            verified = yield verify(delete_passwd)
+            assert isinstance(verified, bool)
+            if verified:
                 defer.returnValue(self.__deleteAccount(profile))
             defer.returnValue(error_ui())
 
@@ -327,16 +440,21 @@ SàT team""" % {'login': login, 'jid': "%s@%s" % (login, self.getConfig('new_acc
             posts = delete_posts_checkbox == 'true'
             comments = delete_comments_checkbox == 'true'
             if posts or comments:
-                if verify(delete_posts_passwd):
+                verified = yield verify(delete_posts_passwd)
+                assert isinstance(verified, bool)
+                if verified:
                     defer.returnValue(self.__deleteBlogPosts(posts, comments, profile))
                 defer.returnValue(error_ui())
+        """
 
         # check for password modification
         current_passwd = data[xml_tools.SAT_FORM_PREFIX + 'current_passwd']
         new_passwd1 = data[xml_tools.SAT_FORM_PREFIX + 'new_passwd1']
         new_passwd2 = data[xml_tools.SAT_FORM_PREFIX + 'new_passwd2']
         if new_passwd1 or new_passwd2:
-            if verify(current_passwd):
+            verified = yield verify(current_passwd)
+            assert isinstance(verified, bool)
+            if verified:
                 if new_passwd1 == new_passwd2:
                     data = yield self.__changePassword(new_passwd1, profile=profile)
                     defer.returnValue(data)
@@ -352,7 +470,7 @@ SàT team""" % {'login': login, 'jid': "%s@%s" % (login, self.getConfig('new_acc
         @param password (str): the new password
         @param profile (str): %(doc_profile)s
         """
-        session_id, dummy = self._sessions.newSession({'new_password': password}, profile)
+        session_id, dummy = self._sessions.newSession({'new_password': password}, profile=profile)
         form_ui = xml_tools.XMLUI("form", title=D_("Change your password?"), submit_id=self.__change_password_id, session_id=session_id)
         form_ui.addText(D_("Note for advanced users: this will actually change both your SàT profile password AND your XMPP account password."))
         form_ui.addText(D_("Continue with changing the password?"))
@@ -481,3 +599,38 @@ SàT team""" % {'login': login, 'jid': "%s@%s" % (login, self.getConfig('new_acc
         d.addCallbacks(deleted, errback)
         return d
 
+    def asyncConnectWithXMPPCredentials(self, jid_s, password):
+        """Create and connect a new SàT profile using the given XMPP credentials.
+
+        Re-use given JID and XMPP password for the profile name and profile password.
+        @param jid_s (unicode): JID
+        @param password (unicode): XMPP password
+        @return Deferred(bool)
+        @raise exceptions.PasswordError, exceptions.ConflictError
+        """
+        try:  # be sure that the profile doesn't exist yet
+            self.host.memory.getProfileName(jid_s)
+        except exceptions.ProfileUnknownError:
+            pass
+        else:
+            raise exceptions.ConflictError
+
+        d = self.createProfile(password, jid_s, jid_s)
+        d.addCallback(lambda dummy: self.host.memory.getProfileName(jid_s))  # checks if the profile has been successfuly created
+        d.addCallback(self.host.asyncConnect, password, 0)
+
+
+        def connected(result):
+            self.sendEmails(None, profile=jid_s)
+            return result
+
+        def removeProfile(failure):  # profile has been successfully created but the XMPP credentials are wrong!
+            log.debug("Removing previously auto-created profile: %s" % failure.getErrorMessage())
+            self.host.memory.asyncDeleteProfile(jid_s)
+            raise failure
+
+        # FIXME: we don't catch the case where the JID host is not an XMPP server, and the user
+        # has to wait until the DBUS timeout ; as a consequence, emails are sent to the admins
+        # and the profile is not deleted. When the host exists, removeProfile is well called.
+        d.addCallbacks(connected, removeProfile)
+        return d
