@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SàT plugin for managing Maildir type mail boxes
@@ -17,26 +17,19 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from sat.core.i18n import _
+from sat.core.i18n import D_, _
 from sat.core.constants import Const as C
 from sat.core.log import getLogger
 log = getLogger(__name__)
 import warnings
 warnings.filterwarnings('ignore', 'the MimeWriter', DeprecationWarning, 'twisted')  # FIXME: to be removed, see http://twistedmatrix.com/trac/ticket/4038
-from twisted.internet import protocol
-from twisted.words.protocols import jabber
-from twisted.cred import portal, checkers
-from twisted.mail import imap4, maildir
-from email.parser import Parser
+from twisted.mail import maildir
 import email.message
-from email.charset import Charset
+import email.utils
 import os
-from cStringIO import StringIO
-from twisted.internet import reactor
 from sat.core.exceptions import ProfileUnknownError
 from sat.memory.persistent import PersistentBinaryDict
 
-from zope.interface import implements
 
 PLUGIN_INFO = {
     "name": "Maildir Plugin",
@@ -50,6 +43,9 @@ PLUGIN_INFO = {
 }
 
 MAILDIR_PATH = "Maildir"
+CATEGORY = D_("Mail Server")
+NAME = D_('Block "normal" messages propagation')
+# FIXME: (very) old and (very) experimental code, need a big cleaning/review
 
 
 class MaildirError(Exception):
@@ -57,17 +53,30 @@ class MaildirError(Exception):
 
 
 class MaildirBox(object):
+    params = """
+    <params>
+    <individual>
+    <category name='{category_name}' label='{category_label}'>
+        <param name='{name}' label='{label}' value="false" type="bool" security="4" />
+    </category>
+    </individual>
+    </params>
+    """.format(category_name=CATEGORY,
+               category_label=_(CATEGORY),
+               name=NAME,
+               label=_(NAME),
+              )
 
     def __init__(self, host):
         log.info(_("Plugin Maildir initialization"))
         self.host = host
+        host.memory.updateParams(self.params)
 
         self.__observed = {}
         self.data = {}  # list of profile spectific data. key = profile, value = PersistentBinaryDict where key=mailbox name,
                      # and value is a dictionnary with the following value
                      #    - cur_idx: value of the current unique integer increment (UID)
                      #    - message_id (as returned by MaildirMailbox): a tuple of (UID, [flag1, flag2, ...])
-        pList = host.memory.getProfilesList  # shorter :)
         self.__mailboxes = {}  # key: profile, value: {boxname: MailboxUser instance}
 
         #the triggers
@@ -100,7 +109,7 @@ class MaildirBox(object):
                 if mess_type != 'normal':
                     return True
                 self.accessMessageBox("INBOX", profile_key=profile).addMessage(message)
-                return False
+                return not self.host.memory.getParamA(NAME, CATEGORY, profile_key=profile)
 
     def accessMessageBox(self, boxname, observer=None, profile_key=C.PROF_KEY_NONE):
         """Create and return a MailboxUser instance
@@ -127,7 +136,7 @@ class MaildirBox(object):
         @param mailboxUser: MailboxUser instance"""
         if boxname not in self.__mailboxes:
             err_msg = _("Trying to remove an mailboxUser not referenced")
-            log.error(_("INTERNAL ERROR: ") + err_msg)
+            log.error(_(u"INTERNAL ERROR: ") + err_msg)
             raise MaildirError(err_msg)
         assert self.__mailboxes[profile][boxname] == mailboxUser
         del self.__mailboxes[profile][boxname]
@@ -146,7 +155,7 @@ class MaildirBox(object):
             return self.data[profile][boxname]  # the boxname MUST exist in the data
         except KeyError:
             err_msg = _("Boxname doesn't exist in internal data")
-            log.error(_("INTERNAL ERROR: ") + err_msg)
+            log.error(_(u"INTERNAL ERROR: ") + err_msg)
             raise MaildirError(err_msg)
 
     def getUid(self, boxname, message_id, profile):
@@ -284,22 +293,22 @@ class MaildirBox(object):
         @param boxname: name of the box which was observed
         @param signal: which signal was observed by the caller"""
         if (profile, boxname) not in self.__observed:
-            err_msg = _("Trying to remove an observer for an inexistant mailbox")
-            log.error(_("INTERNAL ERROR: ") + err_msg)
+            err_msg = _(u"Trying to remove an observer for an inexistant mailbox")
+            log.error(_(u"INTERNAL ERROR: ") + err_msg)
             raise MaildirError(err_msg)
         if signal not in self.__observed[(profile, boxname)]:
-            err_msg = _("Trying to remove an inexistant observer, no observer for this signal")
-            log.error(_("INTERNAL ERROR: ") + err_msg)
+            err_msg = _(u"Trying to remove an inexistant observer, no observer for this signal")
+            log.error(_(u"INTERNAL ERROR: ") + err_msg)
             raise MaildirError(err_msg)
         if not callback in self.__observed[(profile, boxname)][signal]:
-            err_msg = _("Trying to remove an inexistant observer")
-            log.error(_("INTERNAL ERROR: ") + err_msg)
+            err_msg = _(u"Trying to remove an inexistant observer")
+            log.error(_(u"INTERNAL ERROR: ") + err_msg)
             raise MaildirError(err_msg)
         self.__observed[(profile, boxname)][signal].remove(callback)
 
     def emitSignal(self, profile, boxname, signal_name):
         """Emit the signal to observer"""
-        log.debug('emitSignal %s %s %s' % (profile, boxname, signal_name))
+        log.debug(u'emitSignal %s %s %s' % (profile, boxname, signal_name))
         try:
             for observer_cb in self.__observed[(profile, boxname)][signal_name]:
                 observer_cb()
@@ -335,7 +344,7 @@ class MailboxUser(object):
            @param profile: real profile (ie not a profile_key)
            THIS OBJECT MUST NOT BE USED DIRECTLY: use MaildirBox.accessMessageBox instead"""
         if _maildir._checkBoxReference(name, profile):
-            log.error("INTERNAL ERROR: MailboxUser MUST NOT be instancied directly")
+            log.error(u"INTERNAL ERROR: MailboxUser MUST NOT be instancied directly")
             raise MaildirError('double MailboxUser instanciation')
         if name != "INBOX":
             raise NotImplementedError
@@ -353,7 +362,7 @@ class MailboxUser(object):
         self.__uid_table_update()
 
         if observer:
-            log.debug("adding observer for %s (%s)" % (name, profile))
+            log.debug(u"adding observer for %s (%s)" % (name, profile))
             self.maildir.addObserver(observer, profile, name, "NEW_MESSAGE")
 
     def __uid_table_update(self):
@@ -366,7 +375,7 @@ class MailboxUser(object):
 
     def __del__(self):
         if self.observer:
-            log.debug("removing observer for %s" % self.name)
+            log.debug(u"removing observer for %s" % self.name)
             self._maildir.removeObserver(self.observer, self.name, "NEW_MESSAGE")
         self.maildir._removeBoxAccess(self.name, self, profile=self.profile)
 
