@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin for managing various text syntaxes
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -24,8 +24,11 @@ log = getLogger(__name__)
 from twisted.internet import defer
 from twisted.internet.threads import deferToThread
 from sat.core import exceptions
-from lxml import html
-from lxml.html import clean
+try:
+    from lxml import html
+    from lxml.html import clean
+except ImportError:
+    raise exceptions.MissingModule(u"Missing module lxml, please download/install it from http://lxml.de/")
 from cgi import escape
 import re
 
@@ -36,9 +39,10 @@ _SYNTAX_XHTML = "XHTML"
 _SYNTAX_CURRENT = "@CURRENT@"
 
 # TODO: check/adapt following list
-STYLES_WHITELIST = ["azimuth", "background-color", "border-bottom-color", "border-collapse", "border-color", "border-left-color", "border-right-color", "border-top-color", "clear", "color", "cursor", "direction", "display", "elevation", "float", "font", "font-family", "font-size", "font-style", "font-variant", "font-weight", "height", "letter-spacing", "line-height", "overflow", "pause", "pause-after", "pause-before", "pitch", "pitch-range", "richness", "speak", "speak-header", "speak-numeral", "speak-punctuation", "speech-rate", "stress", "text-align", "text-decoration", "text-indent", "unicode-bidi", "vertical-align", "voice-family", "volume", "white-space", "width"] # based on feedparser list (http://pythonhosted.org/feedparser/html-sanitization.html)
+# list initialy based on feedparser list (http://pythonhosted.org/feedparser/html-sanitization.html)
+STYLES_WHITELIST = ("azimuth", "background-color", "border-bottom-color", "border-collapse", "border-color", "border-left-color", "border-right-color", "border-top-color", "clear", "color", "cursor", "direction", "display", "elevation", "float", "font", "font-family", "font-size", "font-style", "font-variant", "font-weight", "height", "letter-spacing", "line-height", "overflow", "pause", "pause-after", "pause-before", "pitch", "pitch-range", "richness", "speak", "speak-header", "speak-numeral", "speak-punctuation", "speech-rate", "stress", "text-align", "text-decoration", "text-indent", "unicode-bidi", "vertical-align", "voice-family", "volume", "white-space", "width")
 
-SAFE_ATTRS = html.defs.safe_attrs.union(('style',))
+SAFE_ATTRS = html.defs.safe_attrs.union(('style', 'poster', 'controls'))
 STYLES_VALUES_REGEX = r'^(' + '|'.join(['([a-z-]+)', # alphabetical names
                                        '(#[0-9a-f]+)', # hex value
                                        '(\d+(.\d+)? *(|%|em|ex|px|in|cm|mm|pt|pc))', # values with units (or not)
@@ -99,6 +103,8 @@ class TextSyntaxes(object):
         self.syntaxes = {}
         self.addSyntax(self.SYNTAX_XHTML, lambda xhtml: defer.succeed(xhtml), lambda xhtml: defer.succeed(xhtml),
                        TextSyntaxes.OPT_NO_THREAD)
+        # TODO: text => XHTML should add <a/> to url like in frontends
+        #       it's probably best to move sat_frontends.tools.strings to sat.tools.common or similar
         self.addSyntax(self.SYNTAX_TEXT, lambda text: escape(text), lambda xhtml: self._removeMarkups(xhtml), [TextSyntaxes.OPT_HIDDEN])
         try:
             import markdown, html2text
@@ -109,12 +115,14 @@ class TextSyntaxes(object):
                 return h.handle(html)
             self.addSyntax(self.SYNTAX_MARKDOWN, markdown.markdown, _html2text, [TextSyntaxes.OPT_DEFAULT])
         except ImportError:
-            log.warning("markdown or html2text not found, can't use Markdown syntax")
+            log.warning(u"markdown or html2text not found, can't use Markdown syntax")
+            log.info(u"You can download/install them from https://pythonhosted.org/Markdown/ and https://github.com/Alir3z4/html2text/")
         host.bridge.addMethod("syntaxConvert", ".plugin", in_sign='sssbs', out_sign='s',
                               async=True, method=self.convert)
 
     def _updateParamOptions(self):
         data_synt = TextSyntaxes.params_data['syntaxes']
+        default_synt = TextSyntaxes.params_data.get('default', _SYNTAX_XHTML)
         syntaxes = []
 
         for syntax in data_synt.keys():
@@ -122,11 +130,11 @@ class TextSyntaxes(object):
             if TextSyntaxes.OPT_HIDDEN not in flags:
                 syntaxes.append(syntax)
 
-        syntaxes.sort(key=unicode.lower)
+        syntaxes.sort(key=lambda synt: synt.lower())
         options = []
 
         for syntax in syntaxes:
-            selected = 'selected="true"' if syntax == _SYNTAX_XHTML else ''
+            selected = 'selected="true"' if syntax == default_synt else ''
             options.append(u'<option value="%s" %s/>' % (syntax, selected))
 
         TextSyntaxes.params_data["options"] = u'\n'.join(options)
@@ -140,7 +148,7 @@ class TextSyntaxes(object):
         """
         return self.host.memory.getParamA(NAME, CATEGORY , profile_key=profile)
 
-    def clean_xhtml(self, xhtml):
+    def cleanXHTML(self, xhtml):
         """ Clean XHTML text by removing potentially dangerous/malicious parts
         @param xhtml: raw xhtml text to clean (or lxml's HtmlElement)
         """
@@ -166,7 +174,7 @@ class TextSyntaxes(object):
                     if value == "none":
                         continue
                     cleaned_styles.append((key, value))
-                return "; ".join(["%s: %s" % (key, value) for key, value in cleaned_styles])
+                return "; ".join(["%s: %s" % (key_, value_) for key_, value_ in cleaned_styles])
 
             if isinstance(xhtml, basestring):
                 xhtml_elt = html.fromstring(xhtml)
@@ -186,13 +194,18 @@ class TextSyntaxes(object):
         return deferToThread(blocking_cleaning, xhtml)
 
     def convert(self, text, syntax_from, syntax_to=_SYNTAX_XHTML, safe=True, profile=None):
-        """ Convert a text between two syntaxes
+        """Convert a text between two syntaxes
+
         @param text: text to convert
         @param syntax_from: source syntax (e.g. "markdown")
         @param syntax_to: dest syntax (e.g.: "XHTML")
         @param safe: clean resulting XHTML to avoid malicious code if True
         @param profile: needed only when syntax_from or syntax_to is set to _SYNTAX_CURRENT
-        @return: converted text """
+        @return(unicode): converted text
+        """
+        # FIXME: convert should be abled to handle domish.Element directly
+        #        when dealing with XHTML
+        # TODO: a way for parser to return parsing errors/warnings
 
         if syntax_from == _SYNTAX_CURRENT:
             syntax_from = self.getCurrentSyntax(profile)
@@ -206,14 +219,14 @@ class TextSyntaxes(object):
         d = None
 
         if TextSyntaxes.OPT_NO_THREAD in syntaxes[syntax_from]["flags"]:
-            d = syntaxes[syntax_from]["to"](text)
+            d = defer.maybeDeferred(syntaxes[syntax_from]["to"], text)
         else:
             d = deferToThread(syntaxes[syntax_from]["to"], text)
 
         #TODO: keep only body element and change it to a div here ?
 
         if safe:
-            d.addCallback(self.clean_xhtml)
+            d.addCallback(self.cleanXHTML)
 
         if TextSyntaxes.OPT_NO_THREAD in syntaxes[syntax_to]["flags"]:
             d.addCallback(syntaxes[syntax_to]["from"])
@@ -225,25 +238,26 @@ class TextSyntaxes(object):
         return d
 
     def addSyntax(self, name, to_xhtml_cb, from_xhtml_cb, flags = None):
-        """ Add a new syntax to the manager
+        """Add a new syntax to the manager
+
         @param name: unique name of the syntax
         @param to_xhtml_cb: callback to convert from syntax to XHTML
         @param from_xhtml_cb: callback to convert from XHTML to syntax
         @param flags: set of optional flags, can be:
             TextSyntaxes.OPT_DEFAULT: use as the default syntax (replace former one)
             TextSyntaxes.OPT_HIDDEN: do not show in parameters
-            TextSyntaxes.OPT_NO_THREAD: do not defer to thread when converting (the callback must then return a deferred)
-
+            TextSyntaxes.OPT_NO_THREAD: do not defer to thread when converting (the callback may then return a deferred)
         """
-        name = unicode(name)
-        flags = flags or []
+        flags = flags if flags is not None else []
         if TextSyntaxes.OPT_HIDDEN in flags and TextSyntaxes.OPT_DEFAULT in flags:
-            raise ValueError("%s and %s are mutually exclusive" % (TextSyntaxes.OPT_HIDDEN, TextSyntaxes.OPT_DEFAULT))
+            raise ValueError(u"{} and {} are mutually exclusive".format(TextSyntaxes.OPT_HIDDEN, TextSyntaxes.OPT_DEFAULT))
 
         syntaxes = TextSyntaxes.params_data['syntaxes']
+        if name in syntaxes:
+            raise exceptions.ConflictError(u"This syntax name already exists: {}".format(name))
         syntaxes[name] = {"to": to_xhtml_cb, "from": from_xhtml_cb, "flags": flags}
         if TextSyntaxes.OPT_DEFAULT in flags:
-            syntaxes = TextSyntaxes.params_data['default'] = name
+            TextSyntaxes.params_data['default'] = name
 
         self._updateParamOptions()
 
