@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin for OTR encryption
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -29,14 +29,15 @@ from sat.tools import xml_tools
 from twisted.words.protocols.jabber import jid
 from twisted.python import failure
 from twisted.internet import defer
-import potr
 from sat.memory import persistent
+import potr
+import copy
 
 NS_OTR = "otr_plugin"
 PRIVATE_KEY = "PRIVATE KEY"
 MAIN_MENU = D_('OTR')
-AUTH_TXT = D_("To authenticate your correspondent, you need to give your below fingerprint *BY AN EXTERNAL CANAL* (i.e. not in this chat), and check that the one he give you is the same as below. If there is a mismatch, there can be a spy between you !")
-DROP_TXT = D_("You private key is used to encrypt messages for your correspondent, nobody except you must know it, if you are in doubt, you should drop it !\n\nAre you sure you want to drop your private key ?")
+AUTH_TXT = D_("To authenticate your correspondent, you need to give your below fingerprint *BY AN EXTERNAL CANAL* (i.e. not in this chat), and check that the one he gives you is the same as below. If there is a mismatch, there can be a spy between you!")
+DROP_TXT = D_("You private key is used to encrypt messages for your correspondent, nobody except you must know it, if you are in doubt, you should drop it!\n\nAre you sure you want to drop your private key?")
 
 DEFAULT_POLICY_FLAGS = {
   'ALLOW_V1':False,
@@ -154,12 +155,12 @@ class Account(potr.context.Account):
         trust_data = self.client.otr_data.get('trust', {})
         for jid_, jid_data in trust_data.iteritems():
             for fingerprint, trust_level in jid_data.iteritems():
-                log.debug('setting trust for {jid}: [{fingerprint}] = "{trust_level}"'.format(jid=jid_, fingerprint=fingerprint, trust_level=trust_level))
+                log.debug(u'setting trust for {jid}: [{fingerprint}] = "{trust_level}"'.format(jid=jid_, fingerprint=fingerprint, trust_level=trust_level))
                 self.trusts.setdefault(jid.JID(jid_), {})[fingerprint] = trust_level
 
     def saveTrusts(self):
-        log.debug("saving trusts for {profile}".format(profile=self.client.profile))
-        log.debug("trusts = {}".format(self.client.otr_data['trust']))
+        log.debug(u"saving trusts for {profile}".format(profile=self.client.profile))
+        log.debug(u"trusts = {}".format(self.client.otr_data['trust']))
         self.client.otr_data.force('trust')
 
     def setTrust(self, other_jid, fingerprint, trustLevel):
@@ -188,7 +189,7 @@ class ContextManager(object):
     def getContextForUser(self, other):
         log.debug(u"getContextForUser [%s]" % other)
         if not other.resource:
-            log.warning("getContextForUser called with a bare jid")
+            log.warning(u"getContextForUser called with a bare jid: %s" % other.full())
         return self.startContext(other)
 
 
@@ -207,7 +208,7 @@ class OTR(object):
         host.importMenu((MAIN_MENU, D_("End session")), self._endSession, security_limit=0, help_string=D_("Finish an OTR session"), type_=C.MENU_SINGLE)
         host.importMenu((MAIN_MENU, D_("Authenticate")), self._authenticate, security_limit=0, help_string=D_("Authenticate user/see your fingerprint"), type_=C.MENU_SINGLE)
         host.importMenu((MAIN_MENU, D_("Drop private key")), self._dropPrivKey, security_limit=0, type_=C.MENU_SINGLE)
-        host.trigger.add("presenceReceived", self.presenceReceivedTrigger)
+        host.trigger.add("presenceReceived", self._presenceReceivedTrigger)
 
     def _fixPotr(self):
         # FIXME: potr fix for bad unicode handling
@@ -269,7 +270,7 @@ class OTR(object):
         try:
             to_jid = jid.JID(menu_data['jid'])
             if not to_jid.resource:
-                to_jid.resource = self.host.memory.getLastResource(to_jid, profile) # FIXME: temporary and unsecure, must be changed when frontends are refactored
+                to_jid.resource = self.host.memory.getMainResource(to_jid, profile) # FIXME: temporary and unsecure, must be changed when frontends are refactored
         except KeyError:
             log.error(_("jid key is not present !"))
             return defer.fail(exceptions.DataError)
@@ -287,7 +288,7 @@ class OTR(object):
         try:
             to_jid = jid.JID(menu_data['jid'])
             if not to_jid.resource:
-                to_jid.resource = self.host.memory.getLastResource(to_jid, profile) # FIXME: temporary and unsecure, must be changed when frontends are refactored
+                to_jid.resource = self.host.memory.getMainResource(to_jid, profile) # FIXME: temporary and unsecure, must be changed when frontends are refactored
         except KeyError:
             log.error(_("jid key is not present !"))
             return defer.fail(exceptions.DataError)
@@ -304,7 +305,7 @@ class OTR(object):
         try:
             to_jid = jid.JID(menu_data['jid'])
             if not to_jid.resource:
-                to_jid.resource = self.host.memory.getLastResource(to_jid, profile) # FIXME: temporary and unsecure, must be changed when frontends are refactored
+                to_jid.resource = self.host.memory.getMainResource(to_jid, profile) # FIXME: temporary and unsecure, must be changed when frontends are refactored
         except KeyError:
             log.error(_("jid key is not present !"))
             return defer.fail(exceptions.DataError)
@@ -329,7 +330,7 @@ class OTR(object):
             # we have a private key, but not the fingerprint of our correspondent
             dialog = xml_tools.XMLUI(C.XMLUI_DIALOG,
                      dialog_opt = {C.XMLUI_DATA_TYPE: C.XMLUI_DIALOG_MESSAGE,
-                                   C.XMLUI_DATA_MESS: _("Your fingerprint is\n{fingerprint}\n\nStart an OTR conversation to have your correspondent one.").format(fingerprint=priv_key),
+                                   C.XMLUI_DATA_MESS: _("Your fingerprint is:\n{fingerprint}\n\nStart an OTR conversation to have your correspondent one.").format(fingerprint=priv_key),
                                    C.XMLUI_DATA_LVL: C.XMLUI_DATA_LVL_INFO
                                   },
                      title = _("Fingerprint"),
@@ -341,10 +342,10 @@ class OTR(object):
             data = xml_tools.XMLUIResult2DataFormResult(raw_data)
             if data['match'] == 'yes':
                 otrctx.setCurrentTrust('verified')
-                note_msg = _("Your correspondant {correspondent} is now TRUSTED")
+                note_msg = _("Your correspondent {correspondent} is now TRUSTED")
             else:
                 otrctx.setCurrentTrust('')
-                note_msg = _("Your correspondant {correspondent} is now UNTRUSTED")
+                note_msg = _("Your correspondent {correspondent} is now UNTRUSTED")
             note = xml_tools.XMLUI(C.XMLUI_DIALOG, dialog_opt = {
                                    C.XMLUI_DATA_TYPE: C.XMLUI_DIALOG_NOTE,
                                    C.XMLUI_DATA_MESS: note_msg.format(correspondent=otrctx.peer)}
@@ -374,7 +375,7 @@ class OTR(object):
         try:
             to_jid = jid.JID(menu_data['jid'])
             if not to_jid.resource:
-                to_jid.resource = self.host.memory.getLastResource(to_jid, profile) # FIXME: temporary and unsecure, must be changed when frontends are refactored
+                to_jid.resource = self.host.memory.getMainResource(to_jid, profile) # FIXME: temporary and unsecure, must be changed when frontends are refactored
         except KeyError:
             log.error(_("jid key is not present !"))
             return defer.fail(exceptions.DataError)
@@ -447,9 +448,9 @@ class OTR(object):
     def sendMessageTrigger(self, mess_data, pre_xml_treatments, post_xml_treatments, profile):
         if profile in self.skipped_profiles:
             return True
-        to_jid = mess_data['to']
+        to_jid = copy.copy(mess_data['to'])
         if mess_data['type'] != 'groupchat' and not to_jid.resource:
-            to_jid.resource = self.host.memory.getLastResource(to_jid, profile) # FIXME: it's dirty, but frontends don't manage resources correctly now, refactoring is planed
+            to_jid.resource = self.host.memory.getMainResource(to_jid, profile) # FIXME: it's dirty, but frontends don't manage resources correctly now, refactoring is planed
         otrctx = self.context_managers[profile].getContextForUser(to_jid)
         if mess_data['type'] != 'groupchat' and otrctx.state != potr.context.STATE_PLAINTEXT:
             if otrctx.state == potr.context.STATE_ENCRYPTED:
@@ -471,11 +472,15 @@ class OTR(object):
             log.debug(u"sending message unencrypted")
             return True
 
-    def presenceReceivedTrigger(self, entity, show, priority, statuses, profile):
-        if show != "unavailable":
-            return
+    def _presenceReceivedTrigger(self, entity, show, priority, statuses, profile):
+        if show != C.PRESENCE_UNAVAILABLE:
+            return True
         if not entity.resource:
-            entity.resource = self.host.memory.getLastResource(entity, profile)  # FIXME: temporary and unsecure, must be changed when frontends are refactored
-        otrctx = self.context_managers[profile].getContextForUser(entity)
-        otrctx.disconnect()
+            try:
+                entity.resource = self.host.memory.getMainResource(entity, profile)  # FIXME: temporary and unsecure, must be changed when frontends are refactored
+            except exceptions.UnknownEntityError:
+                return True  #  entity was not connected
+        if entity in self.context_managers[profile].contexts:
+            otrctx = self.context_managers[profile].getContextForUser(entity)
+            otrctx.disconnect()
         return True
