@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin for Extended Stanza Addressing (xep-0033)
-# Copyright (C) 2013, 2014 Adrien Cossa (souliane@mailoo.org)
+# Copyright (C) 2013-2016 Adrien Cossa (souliane@mailoo.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -33,7 +33,7 @@ except ImportError:
 from twisted.words.xish import domish
 from twisted.internet import defer
 
-from sat.tools.misc import TriggerManager
+from sat.tools import trigger
 from time import time
 
 # TODO: fix Prosody "addressing" plugin to leave the concerned bcc according to the spec:
@@ -72,7 +72,7 @@ class XEP_0033(object):
         log.info(_("Extended Stanza Addressing plugin initialization"))
         self.host = host
         self.internal_data = {}
-        host.trigger.add("sendMessage", self.sendMessageTrigger, TriggerManager.MIN_PRIORITY)
+        host.trigger.add("sendMessage", self.sendMessageTrigger, trigger.TriggerManager.MIN_PRIORITY)
         host.trigger.add("MessageReceived", self.messageReceivedTrigger)
 
     def sendMessageTrigger(self, mess_data, pre_xml_treatments, post_xml_treatments, profile):
@@ -88,8 +88,8 @@ class XEP_0033(object):
                     raise failure.Failure(exceptions.CancelError())
                 if mess_data["to"] not in entities:
                     expected = _(' or ').join([entity.userhost() for entity in entities])
-                    log.warning(_("Stanzas using XEP-0033 should be addressed to %(expected)s, not %(current)s!") % {'expected': expected, 'current': mess_data["to"]})
-                    log.warning(_("TODO: addressing has been fixed by the backend... fix it in the frontend!"))
+                    log.warning(_(u"Stanzas using XEP-0033 should be addressed to %(expected)s, not %(current)s!") % {'expected': expected, 'current': mess_data["to"]})
+                    log.warning(_(u"TODO: addressing has been fixed by the backend... fix it in the frontend!"))
                     mess_data["to"] = list(entities)[0].userhostJID()
                 element = mess_data['xml'].addElement('addresses', NS_ADDRESS)
                 entries = [entry.split(':') for entry in mess_data['extra']['address'].split('\n') if entry != '']
@@ -99,7 +99,7 @@ class XEP_0033(object):
                 self.sendAndStoreMessage(mess_data, entries, profile)
                 log.debug("XEP-0033 took over")
                 raise failure.Failure(exceptions.CancelError())
-            d = self.host.findFeaturesSet([NS_ADDRESS], profile_key=profile)
+            d = self.host.findFeaturesSet([NS_ADDRESS], profile=profile)
             d.addCallbacks(discoCallback, lambda dummy: discoCallback(None))
             return d
 
@@ -154,7 +154,7 @@ class XEP_0033(object):
         defer_list = []
         for type_, jid_ in entries:
             d = defer.Deferred()
-            d.addCallback(self.host.findFeaturesSet, jid_=JID(JID(jid_).host), profile_key=profile)
+            d.addCallback(self.host.findFeaturesSet, jid_=JID(JID(jid_).host), profile=profile)
             d.addCallbacks(discoCallback, errback, callbackArgs=[jid_], errbackArgs=[jid_])
             d.callback([NS_ADDRESS])
             defer_list.append(d)
