@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin for managing xep-0045
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -28,10 +28,13 @@ from sat.core import exceptions
 from sat.memory import memory
 
 import uuid
+import copy
 
-from wokkel import muc
+from wokkel import muc, disco, iwokkel
 from sat.tools import xml_tools
 
+from zope.interface import implements
+
 
 PLUGIN_INFO = {
     "name": "XEP 0045 Plugin",
@@ -45,13 +48,21 @@ PLUGIN_INFO = {
     "description": _("""Implementation of Multi-User Chat""")
 }
 
+NS_MUC = 'http://jabber.org/protocol/muc'
 AFFILIATIONS = ('owner', 'admin', 'member', 'none', 'outcast')
 
+CONFIG_SECTION = u'plugin muc'
+
+default_conf = {"default_muc": u'sat@chat.jabberfr.org'}
+
 
 class UnknownRoom(Exception):
     pass
 
 
+class AlreadyJoinedRoom(Exception):
+    pass
+
 class NotReadyYet(Exception):
     pass
 
@@ -64,13 +75,14 @@ class XEP_0045(object):
         self.host = host
         self.clients = {}
         self._sessions = memory.Sessions()
-        host.bridge.addMethod("joinMUC", ".plugin", in_sign='ssa{ss}s', out_sign='s', method=self._join)
+        host.bridge.addMethod("joinMUC", ".plugin", in_sign='ssa{ss}s', out_sign='s', method=self._join, async=True)
         host.bridge.addMethod("mucNick", ".plugin", in_sign='sss', out_sign='', method=self.mucNick)
         host.bridge.addMethod("mucLeave", ".plugin", in_sign='ss', out_sign='', method=self.mucLeave, async=True)
         host.bridge.addMethod("getRoomsJoined", ".plugin", in_sign='s', out_sign='a(sass)', method=self.getRoomsJoined)
         host.bridge.addMethod("getRoomsSubjects", ".plugin", in_sign='s', out_sign='a(ss)', method=self.getRoomsSubjects)
         host.bridge.addMethod("getUniqueRoomName", ".plugin", in_sign='ss', out_sign='s', method=self._getUniqueName)
         host.bridge.addMethod("configureRoom", ".plugin", in_sign='ss', out_sign='s', method=self._configureRoom, async=True)
+        host.bridge.addMethod("getDefaultMUC", ".plugin", in_sign='', out_sign='s', method=self.getDefaultMUC)
         host.bridge.addSignal("roomJoined", ".plugin", signature='sasss')  # args: room_jid, room_nicks, user_nick, profile
         host.bridge.addSignal("roomLeft", ".plugin", signature='ss')  # args: room_jid, profile
         host.bridge.addSignal("roomUserJoined", ".plugin", signature='ssa{ss}s')  # args: room_jid, user_nick, user_data, profile
@@ -78,18 +90,20 @@ class XEP_0045(object):
         host.bridge.addSignal("roomUserChangedNick", ".plugin", signature='ssss')  # args: room_jid, old_nick, new_nick, profile
         host.bridge.addSignal("roomNewSubject", ".plugin", signature='sss')  # args: room_jid, subject, profile
         self.__submit_conf_id = host.registerCallback(self._submitConfiguration, with_data=True)
-        host.importMenu((D_("MUC"), D_("configure")), self._configureRoomMenu, security_limit=4, help_string=D_("Configure Multi-User Chat room"), type_=C.MENU_ROOM)
+        host.importMenu((D_("MUC"), D_("configure")), self._configureRoomMenu, security_limit=0, help_string=D_("Configure Multi-User Chat room"), type_=C.MENU_ROOM)
         try:
             self.host.plugins[C.TEXT_CMDS].registerTextCommands(self)
             self.host.plugins[C.TEXT_CMDS].addWhoIsCb(self._whois, 100)
         except KeyError:
             log.info(_("Text commands not available"))
 
+        host.trigger.add("presence_available", self.presenceTrigger)
+
     def profileConnected(self, profile):
         def assign_service(service):
             client = self.host.getClient(profile)
             client.muc_service = service
-        return self.getMUCService(profile_key=profile).addCallback(assign_service)
+        return self.getMUCService(profile=profile).addCallback(assign_service)
 
     def checkClient(self, profile):
         """Check if the profile is connected and has used the MUC feature.
@@ -98,7 +112,7 @@ class XEP_0045(object):
         @param profile: profile to check
         @return: True if the profile is connected and has used the MUC feature, else False"""
         if not profile or profile not in self.clients or not self.host.isConnected(profile):
-            log.error(_('Unknown or disconnected profile (%s)') % profile)
+            log.error(_(u'Unknown or disconnected profile (%s)') % profile)
             if profile in self.clients:
                 del self.clients[profile]
             return False
@@ -114,7 +128,7 @@ class XEP_0045(object):
         profile = self.host.memory.getProfileName(profile_key)
         if not self.checkClient(profile):
             raise exceptions.ProfileUnknownError("Unknown or disconnected profile")
-        if room_jid.userhost() not in self.clients[profile].joined_rooms:
+        if room_jid not in self.clients[profile].joined_rooms:
             raise UnknownRoom("This room has not been joined")
         return profile
 
@@ -124,15 +138,13 @@ class XEP_0045(object):
         def _sendBridgeSignal(ignore=None):
             self.host.bridge.roomJoined(room.roomJID.userhost(), [user.nick for user in room.roster.values()], room.nick, profile)
 
-        room_jid_s = room.roomJID.userhost()
-        self.host.memory.updateEntityData(room.roomJID, "type", "chatroom", profile)
-        self.clients[profile].joined_rooms[room_jid_s] = room
+        self.clients[profile].joined_rooms[room.roomJID] = room
         if room.locked:
-            #FIXME: the current behaviour is to create an instant room
-            #and send the signal only when the room is unlocked
-            #a proper configuration management should be done
+            # FIXME: the current behaviour is to create an instant room
+            # and send the signal only when the room is unlocked
+            # a proper configuration management should be done
             print "room locked !"
-            self.clients[profile].configure(room.roomJID, {}).addCallbacks(_sendBridgeSignal, lambda x: log.error(_('Error while configuring the room')))
+            self.clients[profile].configure(room.roomJID, {}).addCallbacks(_sendBridgeSignal, lambda x: log.error(_(u'Error while configuring the room')))
         else:
             _sendBridgeSignal()
         return room
@@ -152,6 +164,16 @@ class XEP_0045(object):
         self.host.bridge.newAlert(mess, D_("Group chat error"), "ERROR", profile)
         raise failure
 
+    def isRoom(self, entity_bare, profile_key):
+        """Tell if a bare entity is a MUC room.
+
+        @param entity_bare (jid.JID): bare entity
+        @param profile_key (unicode): %(doc_profile_key)s
+        @return bool
+        """
+        profile = self.host.memory.getProfileName(profile_key)
+        return entity_bare in self.clients[profile].joined_rooms
+
     def getRoomsJoined(self, profile_key=C.PROF_KEY_NONE):
         """Return room where user is"""
         profile = self.host.memory.getProfileName(profile_key)
@@ -162,27 +184,25 @@ class XEP_0045(object):
             result.append((room.roomJID.userhost(), [user.nick for user in room.roster.values()], room.nick))
         return result
 
-    def getRoomNick(self, room_jid_s, profile_key=C.PROF_KEY_NONE):
+    def getRoomNick(self, room_jid, profile_key=C.PROF_KEY_NONE):
         """return nick used in room by user
 
-        @param room_jid_s: unicode room id
+        @param room_jid (jid.JID): JID of the room
         @profile_key: profile
         @return: nick or empty string in case of error"""
         profile = self.host.memory.getProfileName(profile_key)
-        if not self.checkClient(profile) or room_jid_s not in self.clients[profile].joined_rooms:
+        if not self.checkClient(profile) or room_jid not in self.clients[profile].joined_rooms:
             return ''
-        return self.clients[profile].joined_rooms[room_jid_s].nick
+        return self.clients[profile].joined_rooms[room_jid].nick
 
     def getRoomNickOfUser(self, room, user_jid, secure=True):
         """Returns the nick of the given user in the room.
 
-        @room: instance of wokkel.muc.Room
-        @user: JID or unicode (JID userhost).
-        @param secure: set to True for a secure check
-        @return: the nick or None if the user didn't join the room.
+        @param room (wokkel.muc.Room): the room
+        @param user (jid.JID): bare JID of the user
+        @param secure (bool): set to True for a secure check
+        @return: unicode or None if the user didn't join the room.
         """
-        if not isinstance(user_jid, jid.JID):
-            user_jid = jid.JID(user_jid).userhostJID()
         for user in room.roster.values():
             if user.entity is not None:
                 if user.entity.userhostJID() == user_jid.userhostJID():
@@ -197,11 +217,12 @@ class XEP_0045(object):
     def getRoomNicksOfUsers(self, room, users=[], secure=True):
         """Returns the nicks of the given users in the room.
 
-        @room: instance of wokkel.muc.Room
-        @users: list of JID or unicode (JID userhost).
-        @param secure: set to True for a secure check
-        @return: (x, y) with x a list containing the nicks of
-        the users who are in the room, and y the missing users.
+        @param room (wokkel.muc.Room): the room
+        @param users (list[jid.JID]): list of users
+        @param secure (True): set to True for a secure check
+        @return: a couple (x, y) with:
+            - x (list[unicode]): nicks of the users who are in the room
+            - y (list[jid.JID]): JID of the missing users.
         """
         nicks = []
         missing = []
@@ -229,6 +250,7 @@ class XEP_0045(object):
         except KeyError:
             log.error(_("room_jid key is not present !"))
             return defer.fail(exceptions.DataError)
+
         def xmluiReceived(xmlui):
             return {"xmlui": xmlui.toXml()}
         return self.configureRoom(room_jid, profile).addCallback(xmluiReceived)
@@ -275,7 +297,7 @@ class XEP_0045(object):
     def isNickInRoom(self, room_jid, nick, profile):
         """Tell if a nick is currently present in a room"""
         profile = self.getProfileAssertInRoom(room_jid, profile)
-        return self.clients[profile].joined_rooms[room_jid.userhost()].inRoster(muc.User(nick))
+        return self.clients[profile].joined_rooms[room_jid].inRoster(muc.User(nick))
 
     def getRoomsSubjects(self, profile_key=C.PROF_KEY_NONE):
         """Return received subjects of rooms"""
@@ -285,16 +307,16 @@ class XEP_0045(object):
         return self.clients[profile].rec_subjects.values()
 
     @defer.inlineCallbacks
-    def getMUCService(self, jid_=None, profile_key=C.PROF_KEY_NONE):
+    def getMUCService(self, jid_=None, profile=C.PROF_KEY_NONE):
         """Return first found MUC service of an entity
 
         @param jid_: entity which may have a MUC service, or None for our own server
-        @param profile_key: %(doc_profile_key)s
+        @param profile: %(doc_profile)s
         """
         muc_service = None
-        services = yield self.host.findServiceEntities("conference", "text", jid_, profile_key=profile_key)
+        services = yield self.host.findServiceEntities("conference", "text", jid_, profile=profile)
         for service in services:
-            if not ".irc." in service.userhost():
+            if ".irc." not in service.userhost():
                 # FIXME:
                 # This ugly hack is here to avoid an issue with openfire: the IRC gateway
                 # use "conference/text" identity (instead of "conference/irc")
@@ -308,10 +330,10 @@ class XEP_0045(object):
     def getUniqueName(self, muc_service=None, profile_key=C.PROF_KEY_NONE):
         """Return unique name for a room, avoiding collision
 
-        @param muc_service: leave empty string to use the default service
-        @return: unique room userhost, or '' if an error occured.
+        @param muc_service (jid.JID) : leave empty string to use the default service
+        @return: jid.JID (unique room bare JID)
         """
-        #TODO: we should use #RFC-0045 10.1.4 when available here
+        # TODO: we should use #RFC-0045 10.1.4 when available here
         client = self.host.getClient(profile_key)
         room_name = uuid.uuid1()
         if muc_service is None:
@@ -326,6 +348,13 @@ class XEP_0045(object):
         muc_service = muc_service.userhost()
         return jid.JID("%s@%s" % (room_name, muc_service))
 
+    def getDefaultMUC(self):
+        """Return the default MUC.
+
+        @return: unicode
+        """
+        return self.host.memory.getConfig(CONFIG_SECTION, 'default_muc', default_conf['default_muc'])
+
     def join(self, room_jid, nick, options, profile_key=C.PROF_KEY_NONE):
         def _errDeferred(exc_obj=Exception, txt='Error while joining room'):
             d = defer.Deferred()
@@ -335,12 +364,17 @@ class XEP_0045(object):
         profile = self.host.memory.getProfileName(profile_key)
         if not self.checkClient(profile):
             return _errDeferred()
-        if room_jid.userhost() in self.clients[profile].joined_rooms:
-            log.warning(_('%(profile)s is already in room %(room_jid)s') % {'profile': profile, 'room_jid': room_jid.userhost()})
-            return _errDeferred()
-        log.info(_("[%(profile)s] is joining room %(room)s with nick %(nick)s") % {'profile': profile, 'room': room_jid.userhost(), 'nick': nick})
+        if room_jid in self.clients[profile].joined_rooms:
+            log.warning(_(u'%(profile)s is already in room %(room_jid)s') % {'profile': profile, 'room_jid': room_jid.userhost()})
+            return _errDeferred(AlreadyJoinedRoom, D_(u"The room has already been joined"))
+        log.info(_(u"[%(profile)s] is joining room %(room)s with nick %(nick)s") % {'profile': profile, 'room': room_jid.userhost(), 'nick': nick})
 
-        history_options = options["history"] == "True" if "history" in options else None
+        if "history" in options:
+            history_limit = int(options["history"])
+        else:
+            history_limit = int(self.host.memory.getParamA(C.HISTORY_LIMIT, 'General', profile_key=profile))
+        # http://xmpp.org/extensions/xep-0045.html#enter-managehistory
+        history_options = muc.HistoryOptions(maxStanzas=history_limit)
         password = options["password"] if "password" in options else None
 
         return self.clients[profile].join(room_jid, nick, history_options, password).addCallbacks(self.__room_joined, self.__err_joining_room, callbackKeywords={'profile': profile}, errbackArgs=[room_jid, nick, history_options, password, profile])
@@ -349,25 +383,28 @@ class XEP_0045(object):
         # > /usr/local/lib/python2.7/dist-packages/twisted/internet/defer.py(480)_startRunCallbacks()
         # -> raise AlreadyCalledError(extra)
 
-    def _join(self, room_jid_s, nick, options={}, profile_key=C.PROF_KEY_NONE):
+    def _join(self, room_jid_s, nick, options=None, profile_key=C.PROF_KEY_NONE):
         """join method used by bridge: use the join method, but doesn't return any deferred
-        @return the room userhost (given value or unique generated name)
+        @return: unicode (the room bare)
         """
+        if options is None:
+            options = {}
         profile = self.host.memory.getProfileName(profile_key)
         if not self.checkClient(profile):
             return
-        if room_jid_s == "":
-            room_jid_s = self.getUniqueName(profile_key=profile_key)
-        try:
-            room_jid = jid.JID(room_jid_s)
-        except:
-            mess = _("Invalid room jid: %s") % room_jid_s
-            log.warning(mess)
-            self.host.bridge.newAlert(mess, _("Group chat error"), "ERROR", profile)
-            return
-        self.join(room_jid, nick, options, profile)
+        if room_jid_s:
+            muc_service = self.host.getClient(profile).muc_service
+            try:
+                room_jid = jid.JID(room_jid_s)
+            except (RuntimeError, jid.InvalidFormat, AttributeError):
+                return defer.fail(jid.InvalidFormat(_(u"Invalid room identifier: '%s'. Please give a room short or full identifier like 'room' or 'room@%s'.") % (room_jid_s, unicode(muc_service))))
+            if not room_jid.user:
+                room_jid.user, room_jid.host = room_jid.host, muc_service
+        else:
+            room_jid = self.getUniqueName(profile_key=profile_key)
         # TODO: error management + signal in bridge
-        return room_jid_s
+        d = self.join(room_jid, nick, options, profile)
+        return d.addCallback(lambda room: room.roomJID.userhost())
 
     def nick(self, room_jid, nick, profile_key):
         profile = self.getProfileAssertInRoom(room_jid, profile_key)
@@ -441,44 +478,38 @@ class XEP_0045(object):
     # Text commands #
 
     def cmd_nick(self, mess_data, profile):
-        """change nickname"""
-        log.debug("Catched nick command")
-
-        if mess_data['type'] != "groupchat":
-            self.host.plugins[C.TEXT_CMDS].feedBackWrongContext('nick', 'groupchat', mess_data, profile)
-            return False
+        """change nickname
 
+        @command (group): new_nick
+            - new_nick: new nick to use
+        """
         nick = mess_data["unparsed"].strip()
-        room = mess_data["to"]
-
-        self.nick(room, nick, profile)
+        if nick:
+            room = mess_data["to"]
+            self.nick(room, nick, profile)
 
         return False
 
     def cmd_join(self, mess_data, profile):
-        """join a new room (on the same service if full jid is not specified)"""
-        log.debug("Catched join command")
-
-        if mess_data['type'] != "groupchat":
-            self.host.plugins[C.TEXT_CMDS].feedBackWrongContext('join', 'groupchat', mess_data, profile)
-            return False
+        """join a new room
 
+        @command (all): JID
+            - JID: room to join (on the same service if full jid is not specified)
+        """
         if mess_data["unparsed"].strip():
-            room = self.host.plugins[C.TEXT_CMDS].getRoomJID(mess_data["unparsed"].strip(), mess_data["to"].host)
-            nick = (self.getRoomNick(mess_data["to"].userhost(), profile) or
+            room_jid = self.host.plugins[C.TEXT_CMDS].getRoomJID(mess_data["unparsed"].strip(), mess_data["to"].host)
+            nick = (self.getRoomNick(room_jid, profile) or
                     self.host.getClient(profile).jid.user)
-            self.join(room, nick, {}, profile)
+            self.join(room_jid, nick, {}, profile)
 
         return False
 
     def cmd_leave(self, mess_data, profile):
-        """quit a room"""
-        log.debug("Catched leave command")
-
-        if mess_data['type'] != "groupchat":
-            self.host.plugins[C.TEXT_CMDS].feedBackWrongContext('leave', 'groupchat', mess_data, profile)
-            return False
+        """quit a room
 
+        @command (group): [ROOM_JID]
+            - ROOM_JID: jid of the room to live (current room if not specified)
+        """
         if mess_data["unparsed"].strip():
             room = self.host.plugins[C.TEXT_CMDS].getRoomJID(mess_data["unparsed"].strip(), mess_data["to"].host)
         else:
@@ -489,21 +520,19 @@ class XEP_0045(object):
         return False
 
     def cmd_part(self, mess_data, profile):
-        """just a synonym of /leave"""
+        """just a synonym of /leave
+
+        @command (group): [ROOM_JID]
+            - ROOM_JID: jid of the room to live (current room if not specified)
+        """
         return self.cmd_leave(mess_data, profile)
 
     def cmd_kick(self, mess_data, profile):
         """kick a room member
 
-        @command (group): (nick)
-            - nick: the nick of the person to kick
+        @command (group): ROOM_NICK
+            - ROOM_NICK: the nick of the person to kick
         """
-        log.debug("Catched kick command")
-
-        if mess_data['type'] != "groupchat":
-            self.host.plugins[C.TEXT_CMDS].feedBackWrongContext('kick', 'groupchat', mess_data, profile)
-            return False
-
         options = mess_data["unparsed"].strip().split()
         try:
             nick = options[0]
@@ -516,9 +545,10 @@ class XEP_0045(object):
         d = self.kick(nick, mess_data["to"], {} if len(options) == 1 else {'reason': options[1]}, profile)
 
         def cb(dummy):
-            mess_data['message'] = _('%s has been kicked') % nick
+            feedback_msg = _(u'You have kicked {}').format(nick)
             if len(options) > 1:
-                mess_data['message'] += _(' for the following reason: %s') % options[1]
+                feedback_msg += _(u' for the following reason: {}').format(options[1])
+            self.host.plugins[C.TEXT_CMDS].feedBack(feedback_msg, mess_data, profile)
             return True
         d.addCallback(cb)
         return d
@@ -530,19 +560,13 @@ class XEP_0045(object):
             - JID: the JID of the entity to ban
             - reason: the reason why this entity is being banned
         """
-        log.debug("Catched ban command")
-
-        if mess_data['type'] != "groupchat":
-            self.host.plugins[C.TEXT_CMDS].feedBackWrongContext('ban', 'groupchat', mess_data, profile)
-            return False
-
         options = mess_data["unparsed"].strip().split()
         try:
             jid_s = options[0]
             entity_jid = jid.JID(jid_s).userhostJID()
             assert(entity_jid.user)
             assert(entity_jid.host)
-        except (IndexError, jid.InvalidFormat, AssertionError):
+        except (RuntimeError, jid.InvalidFormat, AttributeError, IndexError, AssertionError):
             feedback = _(u"You must provide a valid JID to ban, like in '/ban contact@example.net'")
             self.host.plugins[C.TEXT_CMDS].feedBack(feedback, mess_data, profile)
             return False
@@ -550,9 +574,10 @@ class XEP_0045(object):
         d = self.ban(entity_jid, mess_data["to"], {} if len(options) == 1 else {'reason': options[1]}, profile)
 
         def cb(dummy):
-            mess_data['message'] = _('%s has been banned') % entity_jid
+            feedback_msg = _(u'You have banned {}').format(entity_jid)
             if len(options) > 1:
-                mess_data['message'] += _(' for the following reason: %s') % options[1]
+                feedback_msg += _(u' for the following reason: {}').format(options[1])
+            self.host.plugins[C.TEXT_CMDS].feedBack(feedback_msg, mess_data, profile)
             return True
         d.addCallback(cb)
         return d
@@ -568,19 +593,13 @@ class XEP_0045(object):
             - none: reset entity privileges
             - outcast: ban entity
         """
-        log.debug("Catched affiliate command")
-
-        if mess_data['type'] != "groupchat":
-            self.host.plugins[C.TEXT_CMDS].feedBackWrongContext('affiliate', 'groupchat', mess_data, profile)
-            return False
-
         options = mess_data["unparsed"].strip().split()
         try:
             jid_s = options[0]
             entity_jid = jid.JID(jid_s).userhostJID()
             assert(entity_jid.user)
             assert(entity_jid.host)
-        except (IndexError, jid.InvalidFormat, AssertionError):
+        except (RuntimeError, jid.InvalidFormat, AttributeError, IndexError, AssertionError):
             feedback = _(u"You must provide a valid JID to affiliate, like in '/affiliate contact@example.net member'")
             self.host.plugins[C.TEXT_CMDS].feedBack(feedback, mess_data, profile)
             return False
@@ -594,19 +613,18 @@ class XEP_0045(object):
         d = self.affiliate(entity_jid, mess_data["to"], {'affiliation': affiliation}, profile)
 
         def cb(dummy):
-            mess_data['message'] = _('New affiliation for %(entity)s: %(affiliation)s') % {'entity': entity_jid, 'affiliation': affiliation}
+            feedback_msg = _(u'New affiliation for %(entity)s: %(affiliation)s').format(entity=entity_jid, affiliation=affiliation)
+            self.host.plugins[C.TEXT_CMDS].feedBack(feedback_msg, mess_data, profile)
             return True
         d.addCallback(cb)
         return d
 
     def cmd_title(self, mess_data, profile):
-        """change room's subject"""
-        log.debug("Catched title command")
-
-        if mess_data['type'] != "groupchat":
-            self.host.plugins[C.TEXT_CMDS].feedBackWrongContext('title', 'groupchat', mess_data, profile)
-            return False
+        """change room's subject
 
+        @command (group): title
+            - title: new room subject
+        """
         subject = mess_data["unparsed"].strip()
 
         if subject:
@@ -616,17 +634,23 @@ class XEP_0045(object):
         return False
 
     def cmd_topic(self, mess_data, profile):
-        """just a synonym of /title"""
+        """just a synonym of /title
+
+        @command (group): title
+            - title: new room subject
+        """
         return self.cmd_title(mess_data, profile)
 
     def _whois(self, whois_msg, mess_data, target_jid, profile):
         """ Add MUC user information to whois """
         if mess_data['type'] != "groupchat":
             return
-        if target_jid.userhost() not in self.clients[profile].joined_rooms:
+        if target_jid.userhostJID() not in self.clients[profile].joined_rooms:
             log.warning(_("This room has not been joined"))
             return
-        user = self.clients[profile].joined_rooms[target_jid.userhost()].getUser(target_jid.resource)
+        if not target_jid.resource:
+            return
+        user = self.clients[profile].joined_rooms[target_jid.userhostJID()].getUser(target_jid.resource)
         whois_msg.append(_("Nickname: %s") % user.nick)
         if user.entity:
             whois_msg.append(_("Entity: %s") % user.entity)
@@ -639,26 +663,66 @@ class XEP_0045(object):
         if user.show:
             whois_msg.append(_("Show: %s") % user.show)
 
+    def presenceTrigger(self, presence_elt, client):
+        # XXX: shouldn't it be done by the server ?!!
+        muc_client = self.clients[client.profile]
+        for room_jid, room in muc_client.joined_rooms.iteritems():
+            elt = copy.deepcopy(presence_elt)
+            elt['to'] = room_jid.userhost() + '/' + room.nick
+            client.presence.send(elt)
+        return True
+
 
 class SatMUCClient (muc.MUCClient):
-    #implements(iwokkel.IDisco)
+    implements(iwokkel.IDisco)
 
     def __init__(self, plugin_parent):
         self.plugin_parent = plugin_parent
         self.host = plugin_parent.host
         muc.MUCClient.__init__(self)
-        self.joined_rooms = {}  # FIXME: seem to do the same thing as MUCClient's _rooms attribute, must be removed
         self.rec_subjects = {}
         self.__changing_nicks = set()  # used to keep trace of who is changing nick,
                                        # and to discard userJoinedRoom signal in this case
         print "init SatMUCClient OK"
 
+    @property
+    def joined_rooms(self):
+        return self._rooms
+
     def subject(self, room, subject):
         return muc.MUCClientProtocol.subject(self, room, subject)
 
+    def availableReceived(self, presence):
+        """
+        Available presence was received.
+        """
+        # XXX: we override MUCClient.availableReceived to fix bugs
+        # (affiliation and role are not set)
+        # FIXME: propose a patch upstream
+
+        room, user = self._getRoomUser(presence)
+
+        if room is None:
+            return
+
+        if user is None:
+            nick = presence.sender.resource
+            user = muc.User(nick, presence.entity)
+
+        # Update user data
+        user.role = presence.role
+        user.affiliation = presence.affiliation
+        user.status = presence.status
+        user.show = presence.show
+
+        if room.inRoster(user):
+            self.userUpdatedStatus(room, user, presence.show, presence.status)
+        else:
+            room.addUser(user)
+            self.userJoinedRoom(room, user)
     def unavailableReceived(self, presence):
-        #XXX: we override this method to manage nickname change
-        #TODO: feed this back to Wokkel
+        # XXX: we override this method to manage nickname change
+        # TODO: feed this back to Wokkel
         """
         Unavailable presence was received.
 
@@ -679,14 +743,12 @@ class SatMUCClient (muc.MUCClient):
             self.__changing_nicks.discard(presence.nick)
             self.userLeftRoom(room, user)
 
-    def receivedGroupChat(self, room, user, body):
-        log.debug('receivedGroupChat: room=%s user=%s body=%s' % (room, user, body))
-
     def userJoinedRoom(self, room, user):
+        self.host.memory.updateEntityData(room.roomJID, "type", "chatroom", profile_key=self.parent.profile)
         if user.nick in self.__changing_nicks:
             self.__changing_nicks.remove(user.nick)
         else:
-            log.debug(_("user %(nick)s has joined room (%(room_id)s)") % {'nick': user.nick, 'room_id': room.occupantJID.userhost()})
+            log.debug(_(u"user %(nick)s has joined room (%(room_id)s)") % {'nick': user.nick, 'room_id': room.occupantJID.userhost()})
             if not self.host.trigger.point("MUC user joined", room, user, self.parent.profile):
                 return
             user_data = {'entity': user.entity.full() if user.entity else '', 'affiliation': user.affiliation, 'role': user.role}
@@ -698,13 +760,12 @@ class SatMUCClient (muc.MUCClient):
         if user.nick == room.nick:
             # we left the room
             room_jid_s = room.roomJID.userhost()
-            log.info(_("Room [%(room)s] left (%(profile)s))") % {"room": room_jid_s,
-                                                             "profile": self.parent.profile})
+            log.info(_(u"Room [%(room)s] left (%(profile)s))") % {"room": room_jid_s,
+                                                                 "profile": self.parent.profile})
             self.host.memory.delEntityCache(room.roomJID, profile_key=self.parent.profile)
-            del self.plugin_parent.clients[self.parent.profile].joined_rooms[room_jid_s]
             self.host.bridge.roomLeft(room.roomJID.userhost(), self.parent.profile)
         else:
-            log.debug(_("user %(nick)s left room (%(room_id)s)") % {'nick': user.nick, 'room_id': room.occupantJID.userhost()})
+            log.debug(_(u"user %(nick)s left room (%(room_id)s)") % {'nick': user.nick, 'room_id': room.occupantJID.userhost()})
             user_data = {'entity': user.entity.full() if user.entity else '', 'affiliation': user.affiliation, 'role': user.role}
             self.host.bridge.roomUserLeft(room.roomJID.userhost(), user.nick, user_data, self.parent.profile)
 
@@ -712,10 +773,26 @@ class SatMUCClient (muc.MUCClient):
         self.host.bridge.roomUserChangedNick(room.roomJID.userhost(), user.nick, new_nick, self.parent.profile)
 
     def userUpdatedStatus(self, room, user, show, status):
-        print("FIXME: MUC status not managed yet")
-        #FIXME:
+        self.host.bridge.presenceUpdate(room.roomJID.userhost() + '/' + user.nick, show or '', 0, {C.PRESENCE_STATUSES_DEFAULT: status or ''}, self.parent.profile)
+
+    def receivedGroupChat(self, room, user, body):
+        log.debug(u'receivedGroupChat: room=%s user=%s body=%s' % (room.roomJID.full(), user, body))
+
+    def receivedHistory(self, room, user, message):
+        # http://xmpp.org/extensions/xep-0045.html#enter-history
+        # log.debug(u'receivedHistory: room=%s user=%s body=%s' % (room.roomJID.full(), user, message))
+        pass
 
     def receivedSubject(self, room, user, subject):
-        log.debug(_("New subject for room (%(room_id)s): %(subject)s") % {'room_id': room.roomJID.full(), 'subject': subject})
+        # http://xmpp.org/extensions/xep-0045.html#enter-subject
+        log.debug(_(u"New subject for room (%(room_id)s): %(subject)s") % {'room_id': room.roomJID.full(), 'subject': subject})
         self.rec_subjects[room.roomJID.userhost()] = (room.roomJID.userhost(), subject)
         self.host.bridge.roomNewSubject(room.roomJID.userhost(), subject, self.parent.profile)
+
+    def getDiscoInfo(self, requestor, target, nodeIdentifier=''):
+        return [disco.DiscoFeature(NS_MUC)]
+
+    def getDiscoItems(self, requestor, target, nodeIdentifier=''):
+        # TODO: manage room queries ? Bad for privacy, must be disabled by default
+        #       see XEP-0045 § 6.7
+        return []
