@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin for Bookmarks (xep-0048)
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -58,7 +58,7 @@ class XEP_0048(object):
         self.host = host
         # self.__menu_id = host.registerCallback(self._bookmarksMenu, with_data=True)
         self.__bm_save_id = host.registerCallback(self._bookmarksSaveCb, with_data=True)
-        host.importMenu((D_("Communication"), D_("bookmarks")), self._bookmarksMenu, security_limit=0, help_string=D_("Use and manage bookmarks"))
+        host.importMenu((D_("Groups"), D_("Bookmarks")), self._bookmarksMenu, security_limit=0, help_string=D_("Use and manage bookmarks"))
         self.__selected_id = host.registerCallback(self._bookmarkSelectedCb, with_data=True)
         host.bridge.addMethod("bookmarksList", ".plugin", in_sign='sss', out_sign='a{sa{sa{ss}}}', method=self._bookmarksList, async=True)
         host.bridge.addMethod("bookmarksRemove", ".plugin", in_sign='ssss', out_sign='', method=self._bookmarksRemove, async=True)
@@ -221,7 +221,7 @@ class XEP_0048(object):
 
         d = self.host.plugins['XEP-0045'].join(room_jid, nick, {}, profile_key=profile)
         def join_eb(failure):
-            log.warning("Error while trying to join room: %s" % failure)
+            log.warning(u"Error while trying to join room: %s" % failure)
             # FIXME: failure are badly managed in plugin XEP-0045. Plugin XEP-0045 need to be fixed before managing errors correctly here
             return {}
         d.addCallbacks(lambda dummy: {}, join_eb)
@@ -422,15 +422,11 @@ class XEP_0048(object):
 
         @command (group): [autojoin | remove]
             - autojoin: join room automatically on connection
+            - remove: remove bookmark(s) for this room
         """
-        log.debug("Catched bookmark command")
         client = self.host.getClient(profile)
         txt_cmd = self.host.plugins[C.TEXT_CMDS]
 
-        if mess_data['type'] != "groupchat":
-            self.host.plugins[C.TEXT_CMDS].feedBackWrongContext('bookmark', 'groupchat', mess_data, profile)
-            return False
-
         options = mess_data["unparsed"].strip().split()
         if options and options[0] not in ('autojoin', 'remove'):
             txt_cmd.feedBack(_("Bad arguments"), mess_data, profile)
