@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin for Ad-Hoc Commands (XEP-0050)
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -75,7 +75,7 @@ class AdHocError(Exception):
         """ Error to be used from callback
         @param error_const: one of XEP_0050.ERROR
         """
-        assert(error_const in XEP_0050.ERROR)
+        assert error_const in XEP_0050.ERROR
         self.callback_error = error_const
 
 class AdHocCommand(XMPPHandler):
@@ -107,12 +107,14 @@ class AdHocCommand(XMPPHandler):
             try:
                 allowed.update(self.client.roster.getJidsFromGroup(group))
             except exceptions.UnknownGroupError:
-                log.warning(_("The groups [%(group)s] is unknown for profile [%(profile)s])" % {'group':group, 'profile':self.client.profile}))
+                log.warning(_(u"The groups [%(group)s] is unknown for profile [%(profile)s])" % {'group':group, 'profile':self.client.profile}))
         if requestor.userhostJID() in allowed:
             return True
         return False
 
     def getDiscoInfo(self, requestor, target, nodeIdentifier=''):
+        if nodeIdentifier != NS_COMMANDS: # FIXME: we should manage other disco nodes here
+            return []
         # identities = [ID_CMD_LIST if self.node == NS_COMMANDS else ID_CMD_NODE] # FIXME
         return [disco.DiscoFeature(NS_COMMANDS)] + self.features
 
@@ -217,7 +219,7 @@ class XEP_0050(object):
                               method=self._requestCommandsList,
                               async=True)
         self.__requesting_id = host.registerCallback(self._requestingEntity, with_data=True)
-        host.importMenu((D_("Service"), D_("commands")), self._commandsMenu, security_limit=4, help_string=D_("Execute ad-hoc commands"))
+        host.importMenu((D_("Service"), D_("Commands")), self._commandsMenu, security_limit=2, help_string=D_("Execute ad-hoc commands"))
 
     def getHandler(self, profile):
         return XEP_0050_handler(self)
@@ -254,7 +256,7 @@ class XEP_0050(object):
             return C.XMLUI_DATA_LVL_WARNING
         else:
             if type_ != 'info':
-                log.warning(_("Invalid note type [%s], using info") % type_)
+                log.warning(_(u"Invalid note type [%s], using info") % type_)
             return C.XMLUI_DATA_LVL_INFO
 
     def _mergeNotes(self, notes):
@@ -399,7 +401,7 @@ class XEP_0050(object):
         form_ui.addText(_("Please enter target jid"), 'instructions')
         form_ui.changeContainer("pairs")
         form_ui.addLabel("jid")
-        form_ui.addString("jid")
+        form_ui.addString("jid", value=self.host.getClient(profile).jid.host)
         return {'xmlui': form_ui.toXml()}
 
     def _statusCallback(self, command_elt, session_data, action, node, profile):
@@ -429,7 +431,7 @@ class XEP_0050(object):
             if show not in SHOWS:
                 raise AdHocError(XEP_0050.ERROR.BAD_PAYLOAD)
             if show == "disconnect":
-               self.host.disconnect(profile)
+                self.host.disconnect(profile)
             else:
                 self.host.setPresence(show=show, profile_key=profile)
 
@@ -460,13 +462,13 @@ class XEP_0050(object):
         d.addCallback(self._items2XMLUI)
         return d
 
-    def addAdHocCommand(self, callback, label, node="", features = None, timeout = 600, allowed_jids = None, allowed_groups = None,
+    def addAdHocCommand(self, callback, label, node=None, features = None, timeout = 600, allowed_jids = None, allowed_groups = None,
                         allowed_magics = None, forbidden_jids = None, forbidden_groups = None, profile_key=C.PROF_KEY_NONE):
         """Add an ad-hoc command for the current profile
 
         @param callback: method associated with this ad-hoc command which return the payload data (see AdHocCommand._sendAnswer), can return a deferred
         @param label: label associated with this command on the main menu
-        @param node: disco item node associated with this command. None or "" to use autogenerated node
+        @param node: disco item node associated with this command. None to use autogenerated node
         @param features: features associated with the payload (list of strings), usualy data form
         @param timeout: delay between two requests before canceling the session (in seconds)
         @param allowed_jids: list of allowed entities
@@ -481,8 +483,7 @@ class XEP_0050(object):
         """
         # FIXME: "@ALL@" for profile_key seems useless and dangerous
 
-        node = node.strip()
-        if not node:
+        if node is None:
             node = "%s_%s" % ('COMMANDS', uuid4())
 
         if features is None:
