@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin for Publish-Subscribe (xep-0071)
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -25,7 +25,10 @@ log = getLogger(__name__)
 from wokkel import disco, iwokkel
 from zope.interface import implements
 # from lxml import etree
-from lxml import html
+try:
+    from lxml import html
+except ImportError:
+    raise exceptions.MissingModule(u"Missing module lxml, please download/install it from http://lxml.de/")
 try:
     from twisted.words.protocols.xmlstream import XMPPHandler
 except ImportError:
@@ -113,8 +116,6 @@ class XEP_0071(object):
             d = self.synt_plg.convert(rich, syntax, self.SYNTAX_XHTML_IM)
             if xhtml:
                 raise exceptions.DataError(_("Can't have xhtml and rich content at the same time"))
-        if xhtml:
-            d = self.synt_plg.clean_xhtml(xhtml)
         d.addCallback(syntax_converted)
         return d
 
