@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT plugin for Chat State Notifications Protocol (xep-0085)
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Adrien Cossa (souliane@mailoo.org)
+# Copyright (C) 2009-2016 Adrien Cossa (souliane@mailoo.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -29,8 +29,9 @@ try:
     from twisted.words.protocols.xmlstream import XMPPHandler
 except ImportError:
     from wokkel.subprotocols import XMPPHandler
-from threading import Timer
 from twisted.words.xish import domish
+from twisted.internet import reactor
+from twisted.internet import error as internet_error
 
 NS_XMPP_CLIENT = "jabber:client"
 NS_CHAT_STATES = "http://jabber.org/protocol/chatstates"
@@ -39,6 +40,7 @@ MESSAGE_TYPES = ["chat", "groupchat"]
 PARAM_KEY = "Notifications"
 PARAM_NAME = "Enable chat state notifications"
 ENTITY_KEY = PARAM_KEY + "_" + PARAM_NAME
+DELETE_VALUE = "DELETE"
 
 PLUGIN_INFO = {
     "name": "Chat State Notifications Protocol Plugin",
@@ -94,6 +96,7 @@ class XEP_0085(object):
     def __init__(self, host):
         log.info(_("Chat State Notifications plugin initialization"))
         self.host = host
+        self.map = {}
 
         # parameter value is retrieved before each use
         host.memory.updateParams(self.params)
@@ -102,7 +105,6 @@ class XEP_0085(object):
         host.trigger.add("MessageReceived", self.messageReceivedTrigger)
         host.trigger.add("sendMessage", self.sendMessageTrigger)
         host.trigger.add("paramUpdateTrigger", self.paramUpdateTrigger)
-        # TODO: handle profile disconnection (free memory in entity data)
 
         # args: to_s (jid as string), profile
         host.bridge.addMethod("chatStateComposing", ".plugin", in_sign='ss',
@@ -114,28 +116,43 @@ class XEP_0085(object):
     def getHandler(self, profile):
         return XEP_0085_handler(self, profile)
 
-    def updateEntityData(self, entity_jid, value, profile):
-        """
-        Update the entity data of the given profile for one or all contacts.
+    def profileDisconnected(self, profile):
+        """Eventually send a 'gone' state to all one2one contacts."""
+        if profile not in self.map:
+            return
+        for to_jid in self.map[profile]:
+            # FIXME: the "unavailable" presence stanza is received by to_jid
+            # before the chat state, so it will be ignored... find a way to
+            # actually defer the disconnection
+            self.map[profile][to_jid]._onEvent('gone')
+        del self.map[profile]
+
+    def updateCache(self, entity_jid, value, profile):
+        """Update the entity data of the given profile for one or all contacts.
         Reset the chat state(s) display if the notification has been disabled.
-        @param entity_jid: contact's JID, or '@ALL@' to update all contacts.
-        @param value: True, False or C.PROF_KEY_NONE to delete the entity data
+
+        @param entity_jid: contact's JID, or C.ENTITY_ALL to update all contacts.
+        @param value: True, False or DELETE_VALUE to delete the entity data
         @param profile: current profile
         """
-        self.host.memory.updateEntityData(entity_jid, ENTITY_KEY, value, profile)
-        if not value or value == C.PROF_KEY_NONE:
-            # disable chat state for this or these contact(s)
+        if value == DELETE_VALUE:
+            self.host.memory.delEntityDatum(entity_jid, ENTITY_KEY, profile)
+        else:
+            self.host.memory.updateEntityData(entity_jid, ENTITY_KEY, value, profile_key=profile)
+        if not value or value == DELETE_VALUE:
+            # reinit chat state UI for this or these contact(s)
             self.host.bridge.chatStateReceived(unicode(entity_jid), "", profile)
 
-    def paramUpdateTrigger(self, name, value, category, type, profile):
-        """
-        Reset all the existing chat state entity data associated with this profile after a parameter modification.
+    def paramUpdateTrigger(self, name, value, category, type_, profile):
+        """Reset all the existing chat state entity data associated with this profile after a parameter modification.
+
         @param name: parameter name
         @param value: "true" to activate the notifications, or any other value to delete it
         @param category: parameter category
+        @param type_: parameter type
         """
         if (category, name) == (PARAM_KEY, PARAM_NAME):
-            self.updateEntityData("@ALL@", True if value == "true" else C.PROF_KEY_NONE, profile)
+            self.updateCache(C.ENTITY_ALL, True if C.bool(value) else DELETE_VALUE, profile=profile)
             return False
         return True
 
@@ -148,22 +165,26 @@ class XEP_0085(object):
             return True
 
         from_jid = JID(message.getAttribute("from"))
-        try:
-            domish.generateElementsNamed(message.elements(), name="body").next()
+        if self._isMUC(from_jid, profile):
+            from_jid = from_jid.userhostJID()
+        else:  # update entity data for one2one chat
+            # assert from_jid.resource # FIXME: assert doesn't work on normal message from server (e.g. server announce), because there is no resource
             try:
-                domish.generateElementsNamed(message.elements(), name="active").next()
-                # contact enabled Chat State Notifications
-                self.updateEntityData(from_jid, True, profile)
+                domish.generateElementsNamed(message.elements(), name="body").next()
+                try:
+                    domish.generateElementsNamed(message.elements(), name="active").next()
+                    # contact enabled Chat State Notifications
+                    self.updateCache(from_jid, True, profile=profile)
+                except StopIteration:
+                    if message.getAttribute('type') == 'chat':
+                        # contact didn't enable Chat State Notifications
+                        self.updateCache(from_jid, False, profile=profile)
+                        return True
             except StopIteration:
-                if message.getAttribute('type') == 'chat':
-                    # contact didn't enable Chat State Notifications
-                    self.updateEntityData(from_jid, False, profile)
-                    return True
-        except StopIteration:
-            pass
+                pass
 
         # send our next "composing" states to any MUC and to the contacts who enabled the feature
-        self.__chatStateInit(from_jid, message.getAttribute("type"), profile)
+        self._chatStateInit(from_jid, message.getAttribute("type"), profile)
 
         state_list = [child.name for child in message.elements() if
                       message.getAttribute("type") in MESSAGE_TYPES
@@ -184,27 +205,44 @@ class XEP_0085(object):
         def treatment(mess_data):
             message = mess_data['xml']
             to_jid = JID(message.getAttribute("to"))
-            if not self.__checkActivation(to_jid, forceEntityData=True, profile=profile):
+            if not self._checkActivation(to_jid, forceEntityData=True, profile=profile):
                 return mess_data
             try:
                 # message with a body always mean active state
                 domish.generateElementsNamed(message.elements(), name="body").next()
                 message.addElement('active', NS_CHAT_STATES)
                 # launch the chat state machine (init the timer)
-                self.__chatStateActive(to_jid, mess_data["type"], profile)
+                if self._isMUC(to_jid, profile):
+                    to_jid = to_jid.userhostJID()
+                self._chatStateActive(to_jid, mess_data["type"], profile)
             except StopIteration:
                 if "chat_state" in mess_data["extra"]:
                     state = mess_data["extra"].pop("chat_state")
-                    assert(state in CHAT_STATES)
+                    assert state in CHAT_STATES
                     message.addElement(state, NS_CHAT_STATES)
             return mess_data
 
         post_xml_treatments.addCallback(treatment)
         return True
 
-    def __checkActivation(self, to_jid, forceEntityData, profile):
+    def _isMUC(self, to_jid, profile):
+        """Tell if that JID is a MUC or not
+
+        @param to_jid (JID): full or bare JID to check
+        @param profile (str): %(doc_profile)s
+        @return: bool
         """
-        @param to_joid: the contact's JID
+        try:
+            type_ = self.host.memory.getEntityDatum(to_jid.userhostJID(), 'type', profile)
+            if type_ == 'chatroom': # FIXME: should not use disco instead ?
+                return True
+        except (exceptions.UnknownEntityError, KeyError):
+            pass
+        return False
+
+    def _checkActivation(self, to_jid, forceEntityData, profile):
+        """
+        @param to_jid: the contact's full JID (or bare if you know it's a MUC)
         @param forceEntityData: if set to True, a non-existing
         entity data will be considered to be True (and initialized)
         @param: current profile
@@ -214,66 +252,70 @@ class XEP_0085(object):
         if not self.host.memory.getParamA(PARAM_NAME, PARAM_KEY, profile_key=profile):
             return False
         # check if notifications should be sent to this contact
+        if self._isMUC(to_jid, profile):
+            return True
+        # FIXME: this assertion crash when we want to send a message to an online bare jid
+        # assert to_jid.resource or not self.host.memory.isEntityAvailable(to_jid, profile) # must either have a resource, or talk to an offline contact
         try:
-            type_ = self.host.memory.getEntityData(to_jid, ['type'], profile)['type']
-            if type_ == 'groupchat':  # always send to groupchat
-                return True
-        except (exceptions.UnknownEntityError, KeyError):
-            pass  # private chat
-        try:
-            return self.host.memory.getEntityData(to_jid, [ENTITY_KEY], profile)[ENTITY_KEY]
+            return self.host.memory.getEntityDatum(to_jid, ENTITY_KEY, profile)
         except (exceptions.UnknownEntityError, KeyError):
             if forceEntityData:
                 # enable it for the first time
-                self.updateEntityData(to_jid, True, profile)
+                self.updateCache(to_jid, True, profile=profile)
                 return True
         # wait for the first message before sending states
         return False
 
-    def __chatStateInit(self, to_jid, mess_type, profile):
+    def _chatStateInit(self, to_jid, mess_type, profile):
         """
         Data initialization for the chat state machine.
+
+        @param to_jid (JID): full JID for one2one, bare JID for MUC
+        @param mess_type (str): "one2one" or "groupchat"
+        @param profile (str): %(doc_profile)s
         """
-        # TODO: use also the resource in map key (not for groupchat)
-        to_jid = to_jid.userhostJID()
         if mess_type is None:
             return
-        if not hasattr(self, "map"):
-            self.map = {}
         profile_map = self.map.setdefault(profile, {})
-        if not to_jid in profile_map:
+        if to_jid not in profile_map:
             machine = ChatStateMachine(self.host, to_jid,
                                        mess_type, profile)
             self.map[profile][to_jid] = machine
 
-    def __chatStateActive(self, to_jid, mess_type, profile_key):
+    def _chatStateActive(self, to_jid, mess_type, profile_key):
         """
         Launch the chat state machine on "active" state.
+
+        @param to_jid (JID): full JID for one2one, bare JID for MUC
+        @param mess_type (str): "one2one" or "groupchat"
+        @param profile (str): %(doc_profile)s
         """
-        # TODO: use also the JID resource in the map key (not for groupchat)
-        to_jid = to_jid.userhostJID()
         profile = self.host.memory.getProfileName(profile_key)
         if profile is None:
             raise exceptions.ProfileUnknownError
-            return
-        self.__chatStateInit(to_jid, mess_type, profile)
+        self._chatStateInit(to_jid, mess_type, profile)
         self.map[profile][to_jid]._onEvent("active")
 
     def chatStateComposing(self, to_jid_s, profile_key):
-        """
-        Move to the "composing" state. Since this method is called
-        from the front-end, it needs to check the values of the
-        parameter "Send chat state notifications" and the entity
+        """Move to the "composing" state when required.
+
+        Since this method is called from the front-end, it needs to check the
+        values of the parameter "Send chat state notifications" and the entity
         data associated to the target JID.
-        TODO: try to optimize this method which is called often
+
+        @param to_jid_s (str): contact full JID as a string
+        @param profile_key (str): %(doc_profile_key)s
         """
-        # TODO: use also the JID resource in the map key (not for groupchat)
-        to_jid = JID(to_jid_s).userhostJID()
+        # TODO: try to optimize this method which is called often
         profile = self.host.memory.getProfileName(profile_key)
         if profile is None:
             raise exceptions.ProfileUnknownError
-            return
-        if not self.__checkActivation(to_jid, forceEntityData=False, profile=profile):
+        to_jid = JID(to_jid_s)
+        if self._isMUC(to_jid, profile):
+            to_jid = to_jid.userhostJID()
+        elif not to_jid.resource:
+            to_jid.resource = self.host.memory.getMainResource(to_jid, profile)
+        if not self._checkActivation(to_jid, forceEntityData=False, profile=profile):
             return
         try:
             self.map[profile][to_jid]._onEvent("composing")
@@ -283,7 +325,7 @@ class XEP_0085(object):
             pass
 
 
-class ChatStateMachine:
+class ChatStateMachine(object):
     """
     This class represents a chat state, between one profile and
     one target contact. A timer is used to move from one state
@@ -311,28 +353,33 @@ class ChatStateMachine:
         notification to the contact (the "active" state is
         automatically sent with each message) and set the timer.
         """
+        assert state in TRANSITIONS
+        transition = TRANSITIONS[state]
+        assert "next_state" in transition and "delay" in transition
+
         if state != self.state and state != "active":
             if state != 'gone' or self.mess_type != 'groupchat':
                 # send a new message without body
-                self.host.sendMessage(self.to_jid, '', '', self.mess_type,
-                                      extra={"chat_state": state},
-                                      profile_key=self.profile)
-        self.state = state
-        if not self.timer is None:
-            self.timer.cancel()
+                log.debug(u"sending state '{state}' to {jid}".format(state=state, jid=self.to_jid.full()))
+                client = self.host.getClient(self.profile)
+                mess_data = {'message': None,
+                             'type': self.mess_type,
+                             'from': client.jid,
+                             'to': self.to_jid,
+                             'subject': None
+                             }
+                self.host.generateMessageXML(mess_data)
+                mess_data['xml'].addElement(state, NS_CHAT_STATES)
+                client.xmlstream.send(mess_data['xml'])
 
-        if not state in TRANSITIONS:
-            return
-        if not "next_state" in TRANSITIONS[state]:
-            return
-        if not "delay" in TRANSITIONS[state]:
-            return
-        next_state = TRANSITIONS[state]["next_state"]
-        delay = TRANSITIONS[state]["delay"]
-        if next_state == "" or delay < 0:
-            return
-        self.timer = Timer(delay, self._onEvent, [next_state])
-        self.timer.start()
+        self.state = state
+        try:
+            self.timer.cancel()
+        except (internet_error.AlreadyCalled, AttributeError):
+            pass
+
+        if transition["next_state"] and transition["delay"] > 0:
+            self.timer = reactor.callLater(transition["delay"], self._onEvent, transition["next_state"])
 
 
 class XEP_0085_handler(XMPPHandler):
