@@ -1,9 +1,9 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
 # SAT: a jabber client
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
-# Copyright (C) 2013, 2014 Adrien Cossa (souliane@mailoo.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2013-2016 Adrien Cossa (souliane@mailoo.org)
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -24,20 +24,21 @@ from constants import Const
 from sat.test import helpers
 from sat.plugins import plugin_xep_0033 as plugin
 from sat.core.exceptions import CancelError
-from sat.core.log import getLogger
-from copy import deepcopy
 from twisted.internet import defer
 from wokkel.generic import parseXml
 from twisted.words.protocols.jabber.jid import JID
 from logging import ERROR
 
-PROFILE = Const.PROFILE[0]
+PROFILE_INDEX = 0
+PROFILE = Const.PROFILE[PROFILE_INDEX]
 JID_STR_FROM = Const.JID_STR[1]
 JID_STR_TO = Const.PROFILE_DICT[PROFILE].host
 JID_STR_X_TO = Const.JID_STR[0]
 JID_STR_X_CC = Const.JID_STR[1]
 JID_STR_X_BCC = Const.JID_STR[2]
 
+ADDRS = ('to', JID_STR_X_TO, 'cc', JID_STR_X_CC, 'bcc', JID_STR_X_BCC)
+
 
 class XEP_0033Test(helpers.SatTestCase):
 
@@ -46,7 +47,7 @@ class XEP_0033Test(helpers.SatTestCase):
         self.plugin = plugin.XEP_0033(self.host)
 
     def test_messageReceived(self):
-        self.host.memory.init()
+        self.host.memory.reinit()
         xml = u"""
         <message type="chat" from="%s" to="%s" id="test_1">
             <body>test</body>
@@ -68,115 +69,118 @@ class XEP_0033Test(helpers.SatTestCase):
             self.assertEqual(data['extra']['addresses'], '%s:%s\n%s:%s\n%s:%s\n' % expected, msg)
 
         treatments.addCallback(cb)
-        treatments.callback(data)
+        return treatments.callback(data)
 
-    def test_sendMessageTrigger(self):
+    def _get_mess_data(self):
         mess_data = {"to": JID(JID_STR_TO),
                      "type": "chat",
                      "message": "content",
                      "extra": {}
                      }
-        addresses = ('to', JID_STR_X_TO, 'cc', JID_STR_X_CC, 'bcc', JID_STR_X_BCC)
-        mess_data["extra"]["address"] = '%s:%s\n%s:%s\n%s:%s\n' % addresses
+        mess_data["extra"]["address"] = '%s:%s\n%s:%s\n%s:%s\n' % ADDRS
         original_stanza = u"""
         <message type="chat" from="%s" to="%s" id="test_1">
             <body>content</body>
         </message>
         """ % (JID_STR_FROM, JID_STR_TO)
         mess_data['xml'] = parseXml(original_stanza.encode("utf-8"))
-        expected = deepcopy(mess_data['xml'])
+        return mess_data
+
+    def _assertAddresses(self, mess_data):
+        """The mess_data that we got here has been modified by self.plugin.sendMessageTrigger,
+        check that the addresses element has been added to the stanza."""
+        expected = self._get_mess_data()['xml']
         addresses_extra = """
         <addresses xmlns='http://jabber.org/protocol/address'>
             <address type='%s' jid='%s'/>
             <address type='%s' jid='%s'/>
             <address type='%s' jid='%s'/>
-        </addresses>""" % addresses
+        </addresses>""" % ADDRS
         addresses_element = parseXml(addresses_extra.encode('utf-8'))
         expected.addChild(addresses_element)
+        self.assertEqualXML(mess_data['xml'].toXml().encode("utf-8"), expected.toXml().encode("utf-8"))
 
-        def assertAddresses(mess_data):
-            """The mess_data that we got here has been modified by self.plugin.sendMessageTrigger,
-            check that the addresses element has been added to the stanza."""
-            self.assertEqualXML(mess_data['xml'].toXml().encode("utf-8"), expected.toXml().encode("utf-8"))
+    def _checkSentAndStored(self):
+        """Check that all the recipients got their messages and that the history has been filled.
+        /!\ see the comments in XEP_0033.sendAndStoreMessage"""
+        sent = []
+        stored = []
+        d_list = []
 
-        def sendMessageErrback(failure, exception_class):
-            """If the failure does encapsulate the expected exception, it will be silently
-            trapped, otherwise it will be re-raised and will make the test fail"""
-            failure.trap(exception_class)
+        def cb(entities, to_jid):
+            if host in entities:
+                if host not in sent:  # send the message to the entity offering the feature
+                    sent.append(host)
+                    stored.append(host)
+                stored.append(to_jid)  # store in history for each recipient
+            else:  # feature not supported, use normal behavior
+                sent.append(to_jid)
+                stored.append(to_jid)
+            helpers.unmuteLogging()
 
-        def checkSentAndStored():
-            """Check that all the recipients got their messages and that the history has been filled.
-            /!\ see the comments in XEP_0033.sendAndStoreMessage"""
-            sent = []
-            stored = []
-            for to_s in (JID_STR_X_TO, JID_STR_X_CC, JID_STR_X_BCC):
-                to_jid = JID(to_s)
-                host = JID(to_jid.host)
-                logger = getLogger()
-                level = logger.getEffectiveLevel()
-                logger.setLevel(ERROR)  # remove log.warning pollution
-                entities = yield self.host.findFeaturesSet([plugin.NS_ADDRESS], jid_=host, profile_key=PROFILE)
-                if host in entities:
-                    if host not in sent:  # send the message to the entity offering the feature
-                        sent.append(host)
-                        stored.append(host)
-                    stored.append(to_jid)  # store in history for each recipient
-                else:  # feature not supported, use normal behavior
-                    sent.append(to_jid)
-                    stored.append(to_jid)
-                logger.setLevel(level)
+        for to_s in (JID_STR_X_TO, JID_STR_X_CC, JID_STR_X_BCC):
+            to_jid = JID(to_s)
+            host = JID(to_jid.host)
+            helpers.muteLogging()
+            d = self.host.findFeaturesSet([plugin.NS_ADDRESS], jid_=host, profile=PROFILE)
+            d.addCallback(cb, to_jid)
+            d_list.append(d)
+
+        def cb_list(dummy):
             msg = "/!\ see the comments in XEP_0033.sendAndStoreMessage"
-            self.assertEqualUnsortedList(self.host.sent_messages, sent, msg)
+            sent_recipients = [JID(elt['to']) for elt in self.host.getSentMessages(PROFILE_INDEX)]
+            self.assertEqualUnsortedList(sent_recipients, sent, msg)
             self.assertEqualUnsortedList(self.host.stored_messages, stored, msg)
 
-        def trigger(data, exception):
-            """Execute self.plugin.sendMessageTrigger with a different logging
-            level to not pollute the output, then check that the plugin did its
-            job. It should abort sending the message or add the extended
-            addressing information to the stanza.
-            @param data: the data to be processed by self.plugin.sendMessageTrigger
-            @param exception: CancelError
-            """
-            logger = getLogger()
-            level = logger.getEffectiveLevel()
-            logger.setLevel(ERROR)  # remove log.warning pollution
-            pre_treatments = defer.Deferred()
-            post_treatments = defer.Deferred()
-            self.plugin.sendMessageTrigger(data, pre_treatments, post_treatments, PROFILE)
-            post_treatments.callback(data)
-            logger.setLevel(level)
-            post_treatments.addCallbacks(assertAddresses, lambda failure: sendMessageErrback(failure, exception))
+        return defer.DeferredList(d_list).addCallback(cb_list)
 
+    def _trigger(self, data):
+        """Execute self.plugin.sendMessageTrigger with a different logging
+        level to not pollute the output, then check that the plugin did its
+        job. It should abort sending the message or add the extended
+        addressing information to the stanza.
+        @param data: the data to be processed by self.plugin.sendMessageTrigger
+        """
+        pre_treatments = defer.Deferred()
+        post_treatments = defer.Deferred()
+        helpers.muteLogging()
+        self.plugin.sendMessageTrigger(data, pre_treatments, post_treatments, PROFILE)
+        post_treatments.callback(data)
+        helpers.unmuteLogging()
+        post_treatments.addCallbacks(self._assertAddresses, lambda failure: failure.trap(CancelError))
+        return post_treatments
+
+    def test_sendMessageTriggerFeatureNotSupported(self):
         # feature is not supported, abort the message
-        self.host.memory.init()
-        data = deepcopy(mess_data)
-        trigger(data, CancelError)
+        self.host.memory.reinit()
+        data = self._get_mess_data()
+        return self._trigger(data)
 
+    def test_sendMessageTriggerFeatureSupported(self):
         # feature is supported by the main target server
-        self.host.init()
-        self.host.memory.init()
+        self.host.reinit()
         self.host.addFeature(JID(JID_STR_TO), plugin.NS_ADDRESS, PROFILE)
-        data = deepcopy(mess_data)
-        trigger(data, CancelError)
-        checkSentAndStored()
+        data = self._get_mess_data()
+        d = self._trigger(data)
+        return d.addCallback(lambda dummy: self._checkSentAndStored())
 
+    def test_sendMessageTriggerFeatureFullySupported(self):
         # feature is supported by all target servers
-        self.host.init()
-        self.host.memory.init()
+        self.host.reinit()
         self.host.addFeature(JID(JID_STR_TO), plugin.NS_ADDRESS, PROFILE)
         for dest in (JID_STR_X_TO, JID_STR_X_CC, JID_STR_X_BCC):
             self.host.addFeature(JID(JID(dest).host), plugin.NS_ADDRESS, PROFILE)
-        data = deepcopy(mess_data)
-        trigger(data, CancelError)
-        checkSentAndStored()
+        data = self._get_mess_data()
+        d = self._trigger(data)
+        return d.addCallback(lambda dummy: self._checkSentAndStored())
 
+    def test_sendMessageTriggerFixWrongEntity(self):
         # check that a wrong recipient entity is fixed by the backend
-        self.host.init()
-        self.host.memory.init()
+        self.host.reinit()
         self.host.addFeature(JID(JID_STR_TO), plugin.NS_ADDRESS, PROFILE)
         for dest in (JID_STR_X_TO, JID_STR_X_CC, JID_STR_X_BCC):
             self.host.addFeature(JID(JID(dest).host), plugin.NS_ADDRESS, PROFILE)
-        data = deepcopy(mess_data)
+        data = self._get_mess_data()
         data["to"] = JID(JID_STR_X_TO)
-        trigger(data, CancelError)
-        checkSentAndStored()
+        d = self._trigger(data)
+        return d.addCallback(lambda dummy: self._checkSentAndStored())
